#ifndef WIN32_LEAN_AND_MEAN			//prevents windows.h from including winsock.h
#define WIN32_LEAN_AND_MEAN
#endif

#define PLUGIN_IP_DEFAULT "127.0.0.1"

//#include <windows.h>

#include "CableDetector.h"
#include "CommonCode\RTOS abstraction layer\config.h"

//#include <stdio.h>
//#include <iostream>

#include <tchar.h>
//#include <string.h>

CCableDetector::CCableDetector() :
	m_navPort(8886),
	m_ssPort(8887),
	m_useCanny(true),
	m_cannyMinTres(100),
	m_cannyMaxTres(300),
	m_houghTransform(2),
	m_houghTreshold(50),
	m_houghMinLineLength(50),
	m_houghMaxLineGap(10),
	m_arraysIncluded(100),
	m_thetaResolution(180),
	m_rhoResolution(1),
	m_pauseMillisec(30),
	m_skipArrays(0),
	m_accuReduction(0),
	m_TVGMultiplier(100),
	m_maxAngRate(0.1)
{
	readConfig();

	//char waypointFile[128];
	//GetConfigStringFromFile("test.ini", "Guidance", "WaypointFile", "WaypointFile", waypointFile, sizeof(waypointFile));

	m_pNavThread = new CNavThread(this);
	m_pSideScanThread = new CSideScanThread(this);
	m_pImageProcessor = new ImageProcessor(this);
}

CCableDetector::~CCableDetector()
{
	if (m_pNavThread != NULL)
	{
		delete(m_pNavThread);
		m_pNavThread = NULL;
	}

	if (m_pSideScanThread != NULL)
	{
		delete(m_pSideScanThread);
		m_pSideScanThread = NULL;
	}

	if (m_pImageProcessor != NULL)
	{
		delete(m_pImageProcessor);
		m_pImageProcessor = NULL;
	}
}

void main()
{
	CCableDetector *cableDetector;
	cableDetector = new CCableDetector;

	cableDetector->activate();
	cableDetector->wait();

	delete(cableDetector);
}

void CCableDetector::activate()
{
	m_pNavThread->startThread();
	m_pSideScanThread->startThread();
	m_pImageProcessor->startThread();
}

void CCableDetector::wait()
{
	HANDLE myhandle[3];

	myhandle[0] = m_pNavThread->navHandle;
	myhandle[1] = m_pSideScanThread->ssHandle;
	myhandle[2] = m_pImageProcessor->imgHandle;

	while (true)
	{
		std::cout << "Input cable treshold:" << std::endl;
		std::cin >> m_cableTres;

		//std::cout << "Input hough treshold:" << std::endl;
		//std::cin >> m_houghTreshold;
	}

	WaitForMultipleObjects(3, myhandle, true, INFINITE);
}

void CCableDetector::readConfig()
{
	strcpy_s(m_cableTrackerIP, sizeof(m_cableTrackerIP), PLUGIN_IP_DEFAULT);
	std::string line;
	std::ifstream myfile("CableDetector.ini");
	std::string key;

	std::cout << "CCableDetector: Reading file: CableDetector.ini" << std::endl;

	if (myfile.is_open())
	{
		while (getline(myfile, line))
		{
			std::istringstream is_line(line);
			std::string key;

			if (std::getline(is_line, key, '='))
			{
				std::string value;
				if (std::getline(is_line, value,'\t'))
				{
					if (key.compare(0, 7, "NavPort") == 0)
					{
						m_navPort = stoi(value);
					}
					else if (key.compare(0, 12, "SidescanPort") == 0)
					{
						m_ssPort = stoi(value);
					}
					else if (key.compare(0, 14, "CableTrackerIP") == 0)
					{
						strcpy_s(m_cableTrackerIP, sizeof(m_cableTrackerIP), value.c_str());
					}
					else if (key.compare(0, 9, "CableTres") == 0)
					{
						m_cableTres = stoi(value);
					}
					else if (key.compare(0, 8, "UseCanny") == 0)
					{
						m_useCanny = stoi(value) == 1;
					}
					else if (key.compare(0, 12, "CannyMinTres") == 0)
					{
						m_cannyMinTres = stoi(value);
					}
					else if (key.compare(0, 12, "CannyMaxTres") == 0)
					{
						m_cannyMaxTres = stoi(value);
					}
					else if (key.compare(0, 14, "HoughTransform") == 0)
					{
						m_houghTransform = stoi(value);
					}
					else if (key.compare(0, 13, "HoughTreshold") == 0)
					{
						m_houghTreshold = stoi(value);
					}
					else if (key.compare(0, 18, "HoughMinLineLength") == 0)
					{
						m_houghMinLineLength = stoi(value);
					}
					else if (key.compare(0, 15, "HoughMaxLineGap") == 0)
					{
						m_houghMaxLineGap = stoi(value);
					}
					else if (key.compare(0, 14, "ArraysIncluded") == 0)
					{
						m_arraysIncluded = stoi(value);
					}
					else if (key.compare(0, 15, "ThetaResolution") == 0)
					{
						m_thetaResolution = stoi(value);
					}
					else if (key.compare(0, 13, "RhoResolution") == 0)
					{
						m_rhoResolution = stoi(value);
					}
					else if (key.compare(0, 13, "PauseMillisec") == 0)
					{
						m_pauseMillisec = stoi(value);
					}
					else if (key.compare(0, 10, "SkipArrays") == 0)
					{
						m_skipArrays = stoi(value);
					}
					else if (key.compare(0, 20, "AccumulatorReduction") == 0)
					{
						m_accuReduction = stod(value);
					}
					else if (key.compare(0, 10, "SaveImages") == 0)
					{
						m_bSaveImages = stoi(value) == 1;
					}
					else if (key.compare(0, 13, "TVGMultiplier") == 0)
					{
						m_TVGMultiplier = stod(value);
					}
					else if (key.compare(0, 10, "MaxAngRate") == 0)
					{
						m_maxAngRate = stod(value);
					}
					else
					{
						std::cout << "CCableDetector::readConfig: Undefined key: " << key << std::endl;
					}
				}
			}
		}
		myfile.close();
	}
	else std::cout << "CCableDetector::readConfig: Unable to open ini file" << std::endl;
}

void CCableDetector::gpps()
{
	TCHAR   inBuf[80];
	//char pIP[128];

	GetPrivateProfileString(TEXT("CableTrackerPlugin"),
		TEXT("UdpPort"),
		TEXT("8887"),
		inBuf,
		6,
		TEXT("C:/Users/Eirik/Dropbox/Koding/VisualStudio2015/CableDetector/CableDetector/CableDetector.ini"));
	
	int udpPort = _tstoi(inBuf);

	std::cout << "CCableDetector::gpps: UdpPort = " << udpPort << std::endl;

	GetPrivateProfileString(TEXT("CableTrackerPlugin"),
		TEXT("ReconIP"),
		TEXT(PLUGIN_IP_DEFAULT),
		inBuf,
		16,
		TEXT("C:/Users/Eirik/Dropbox/Koding/VisualStudio2015/CableDetector/CableDetector/CableDetector.ini"));

	//char* pIP = (char*)inBuf;

	//std::string a = inBuf;

	//char * p = strtok_s(inBuf, ",");

	//MultiByteToWideChar(CP_ACP, MB_COMPOSITE, inBuf, -1, pIP, 128);

	//std::cout << "ip: " << a << "." << std::endl;
}