
#include "ImageProcessor.h"
#include "CableDetector.h"

#include <math.h>
#include <iomanip>      // std::setprecision

/// Function headers
//int display_caption(char* caption);
///int display_dst(int delay);
//void showImage();

ImageProcessor::ImageProcessor(CCableDetector *pParent) :
	m_pParent(pParent),
	m_bRunning(false),
	m_startLine(1),
	m_rowStart(0),
	m_accumulatorHeight(0),
	m_accumulatorWidth(180),
	m_messageNumber(0),
	m_linesWritten(0),
	m_cableHeading(0),
	m_crossTrackError(0),
	m_scaleImage(true),
	m_meterPerPixel(0.1)
{
	m_accumulatorWidth = m_pParent->m_thetaResolution;

	sinTable = new double[m_accumulatorWidth];
	cosTable = new double[m_accumulatorWidth];
	for (int i = 0; i < m_accumulatorWidth; i++)
	{
		sinTable[i] = sin(i*CV_PI / m_accumulatorWidth);
		cosTable[i] = cos(i*CV_PI / m_accumulatorWidth);
	}
}

ImageProcessor::~ImageProcessor()
{
	delete(sinTable);
	delete(cosTable);
	CloseHandle(imgHandle);
}

DWORD ImageProcessor::main()
{
	//int skipLines = m_pParent->m_skipArrays;
	int sleepMs = m_pParent->m_pauseMillisec;

	Sleep(2000);

	while (true)
	{
		//if (m_pParent->getSideScanThread()->getLinesWritten() > m_linesWritten + skipLines)
		if (m_pParent->getSideScanThread()->getLinesWritten() > m_linesWritten) // && (m_pParent->getSideScanThread()->getLinesWritten() % 1 == 0 ))
		{
			m_linesWritten = m_pParent->getSideScanThread()->getLinesWritten();
			if (readData())
			{
				lineDetection();
				Sleep(sleepMs);
			}
		}
		else
			Sleep(sleepMs);
	}
}

unsigned int __stdcall imgThreadStart(void* data)
{
	//printf("SideScanThread: _beginthreadex thread id: %d \n", GetCurrentThreadId());
	ImageProcessor* This = (ImageProcessor*)data;
	return This->main();
}

bool ImageProcessor::startThread()
{
	// Check if module has been initialized, if not, initialize module now
	if (m_bRunning)
		return m_bRunning;

	imgHandle = (HANDLE)_beginthreadex(0, 0, &imgThreadStart, (void*) this, 0, 0);

	m_bRunning = true;

	return m_bRunning;
}

bool ImageProcessor::readData()
{
	int sidescanArrayLength = m_pParent->getSideScanThread()->getSidscanArrayLength();
	int arraysPerImage = m_pParent->m_arraysIncluded;

	int startReadLine = m_linesWritten > arraysPerImage ? m_linesWritten - arraysPerImage + 1 : 1;
	int imgHeight = m_linesWritten > arraysPerImage ? arraysPerImage : m_linesWritten;
	std::string line;
	unsigned int pingNum = 0;
	//float speed = 0;
	int lineNum = imgHeight-1, i = 0;
	int l_arrays = 0;
	double value = 0;

	data1 = new double[imgHeight*sidescanArrayLength * 2];
	
	std::ifstream readFile(m_pParent->getSideScanThread()->getOutputFile());
	gotoLine(readFile, startReadLine);

	if (readFile.is_open())
	{
		if (m_linesWritten > 0)
		{
			while (readFile >> value && lineNum >=0)
			{
				if (i == 0)
				{
					//timestamp
				}
				else if (i == 1)
				{
					pingNum = (uint) value;
				}
				else if (i < 7)
				{
					//other values
				}
				else if (i == 7)
				{
					l_arrays = ((int) value) * 2;
					if (l_arrays < 1)
					{
						lineNum--;
						i = -1;
					}
				}
				else if (i < 7 + l_arrays) {
					//data[lineNum][i - 8] = value;
					data1[lineNum*l_arrays + i - 8] = value/128;
				}
				else if (i == 7 + l_arrays) {
					//data[lineNum][i - 8] = value;
					data1[lineNum*l_arrays + i - 8] = value/128;
					lineNum--;
					i = -1;
				}

				i++;
			}
		}
		else
		{
			return false;
		}
	}
	else
	{
		//std::cout << "CableDetector::ImageProcessor: Failed to open file " << std::endl;
		readFile.close();
		return false;
	}

	readFile.close();

	//std::cout << "lines written: " << m_linesWritten << "  ping number: " << pingNum << "  l_arrays" << l_arrays  << std::endl;

	//src = cv::Mat(imgHeight, l_arrays, CV_64FC1, &data);

	src = cv::Mat(imgHeight, sidescanArrayLength * 2, CV_64FC1, data1);

	if (m_pParent->m_bSaveImages)
		imwrite("./sidescan.bmp", src);
	
	return true;
}

std::ifstream& ImageProcessor::gotoLine(std::ifstream& file, unsigned int num) {

	file.seekg(std::ios::beg);
	for (uint i = 0; i < num - 1; ++i) {
		file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	}
	return file;
}

void ImageProcessor::lineDetection() {
	LARGE_INTEGER frequency;        // ticks per second
	LARGE_INTEGER t1, t2, t3, t4;           // ticks
	double filterTime, edgeTime, houghTime, totalTime;
	QueryPerformanceFrequency(&frequency);

	cv::Mat src_filt, src_gray, edge, hough, weight, src_tres;
	int closestLine = -1, yMin = 100000;
	bool saveImage = m_pParent->m_bSaveImages;
	int cableTres = m_pParent->m_cableTres;

	m_crossTrackError = NAN;
	m_cableHeading = NAN;

	/*
		int rowsIncluded = 200;
		std::string filename = "parallel_cables3.bmp";
		cv::Mat src_large = cv::imread(filename, 0);

		rowStart = (rowStart + rowsIncluded) >= src_large.rows ? src_large.rows - rowsIncluded : rowStart;

		if ((rowStart + rowsIncluded) >= src_large.rows)
		{
			rowStart = src_large.rows - rowsIncluded;
			delete(data1);
			return;
		}

		src_large(cv::Rect(0, rowStart, src_large.cols, rowsIncluded)).copyTo(src);
	*/

	if (src.empty() || src.rows < 3)
	{
		//std::cout << "ImageProcessor: Cannot find sidescan data" << std::endl; //<< filename << std::endl;
		delete(data1);
		return;
	}

	applyTVG(src);

	if (m_pParent->m_bSaveImages)
		imwrite("./srcTVG.bmp", src);
	
	QueryPerformanceCounter(&t1);

	GaussianBlur(src, src_filt, cv::Size(3, 3), 0, 0, cv::BORDER_DEFAULT);
	//blur(src, src_filt, cv::Size(3, 3), cv::Point(-1, -1));

	//src.copyTo(src_filt);

	delete(data1);
	
	if (m_pParent->m_bSaveImages)
		imwrite("filtered.bmp", src_filt);

	src_filt.convertTo(src_filt, CV_8UC1);

	if (m_scaleImage);
		scaleImage(src_filt);

	bitwise_not(src_filt, src_tres);
	threshold(src_tres, src_tres, m_pParent->m_cableTres, 255, 0);

	//imwrite("tresholded.bmp", src_tres);

	QueryPerformanceCounter(&t2);

	if (m_pParent->m_useCanny)
	{
		cv::Canny(src_filt, edge, m_pParent->m_cannyMinTres, m_pParent->m_cannyMaxTres, 3, false);

		if (saveImage)
			imwrite("./canny_image.bmp", edge);

		edge = edge - src_tres;

		if (saveImage)
			imwrite("./canny_tresholded.bmp", edge);
	}
	else
	{
		int scale = 1;
		int delta = 0;
		int ddepth = CV_8UC1; // CV_16S; //
		bool bLaplace = false;
		double thresh;

		if (bLaplace)
		{
			thresh = 20;
			cv::Laplacian(src_filt, edge, ddepth, 1, 1, 0, 4);
		}
		else
		{
			cv::Mat grad_x, grad_y;
			cv::Mat abs_grad_x, abs_grad_y;

			// Gradient X
			cv::Sobel(src_filt, grad_x, ddepth, 1, 0, 3, scale, delta, cv::BORDER_DEFAULT);
			//cv::Scharr(src_filt, grad_x, ddepth, 1, 0, scale, delta, cv::BORDER_DEFAULT);

			// Gradient Y
			cv::Sobel(src_filt, grad_y, ddepth, 0, 1, 3, scale, delta, cv::BORDER_DEFAULT);
			//cv::Scharr(src_filt, grad_y, ddepth, 0, 1, scale, delta, cv::BORDER_DEFAULT);

			cv::convertScaleAbs(grad_x, abs_grad_x);
			cv::convertScaleAbs(grad_y, abs_grad_y);

			cv::addWeighted(abs_grad_x, 0.5, abs_grad_y, 0.5, 0, edge);

			thresh = 100;
		}
		
		double maxval = 0;

		if (saveImage)
			imwrite("./grad_image.bmp", edge);

		threshold(edge, edge, thresh, maxval, CV_THRESH_TOZERO);

		if (saveImage)
			imwrite("./grad_image_tresholded.bmp", edge);

		cv::subtract(edge, src_tres, edge, cv::Mat(), CV_8UC1);

		//if (saveImage)
		//	imwrite("./grad_image_tresholded2.bmp", edge);
	}

	//remove water colum edges AUV
	//edge(cv::Rect((int)round(edge.cols*0.4), 0, (int)round(edge.cols*0.2), edge.rows)) = cv::Scalar(0, 0, 0);

	int edgePoints = 0;

	for (int y = 0; y < edge.rows; y++)
	{
		for (int x = 0; x < edge.cols; x++)
		{
			if (edge.at<bool>(y, x) != 0)
				edgePoints++;
		}
	}

	//double points = cv::sum(edge)[0];
	
	QueryPerformanceCounter(&t3);
	
	//switch to BGR image
	cvtColor(edge, hough, cv::COLOR_GRAY2BGR);
	
	unsigned int maxAccumulation;

	if (m_pParent->m_houghTransform == 0)
	{
		maxAccumulation = houghTransform(edge, m_pParent->m_houghTreshold);

		if (!saveImage)
			delete(houghAccumulator);
		else
		{
			bitwise_not(hough, hough);
			int drawLines = maxAccumulation > (unsigned int) m_pParent->m_houghTreshold ? maxAccumulation : m_pParent->m_houghTreshold;
			std::vector< std::pair< std::pair<int, int>, std::pair<int, int> > > lines = getLines(drawLines);

			std::vector< std::pair< std::pair<int, int>, std::pair<int, int> > >::iterator it;

			for (it = lines.begin(); it != lines.end(); it++)
			{
				cv::line(hough, cv::Point(it->first.first, it->first.second), cv::Point(it->second.first, it->second.second), cv::Scalar(0, 0, 255), 2, 8);
			}
			
			m_linesFound = lines.size();
			//imwrite("./standard_hough.bmp", hough);
		}
	}
	else if (m_pParent->m_houghTransform == 1)
	{
		std::vector<cv::Vec2f> lines;
		cv::HoughLines(edge, lines, m_pParent->m_rhoResolution, CV_PI / m_accumulatorWidth, m_pParent->m_houghTreshold, 0, 0);

		m_linesFound = lines.size();

		for (size_t i = 0; i < lines.size(); i++)
		{
			float rho = lines[i][0], theta = lines[i][1];
			
			double a = cos(theta), b = sin(theta);
			double x0 = a*rho, y0 = b*rho;

			//ignore orthogonal lines
			if (theta > 0.9*CV_PI / 2 && theta < 1.1*CV_PI / 2)
			{
				//line(hough, pt1, pt2, cv::Scalar(0, 255, 255), 2, cv::LINE_AA);
			}
			else
			{
				double crossTrackError = (x0 - edge.cols / 2)*a + y0*b;
				double cableAngle = theta;

				if (theta > CV_PI / 2)
				{
					crossTrackError = -crossTrackError;
					cableAngle -= CV_PI;
				}

				estimateCablePose(crossTrackError, cableAngle, edge.cols / 2);
				sendResult();

				if (saveImage)
				{
					cv::Point pt1, pt2;

					pt1.x = cvRound(x0 + 1000 * (-b));
					pt1.y = cvRound(y0 + 1000 * (a));
					pt2.x = cvRound(x0 - 1000 * (-b));
					pt2.y = cvRound(y0 - 1000 * (a));

					line(hough, pt1, pt2, cv::Scalar(0, 0, 255), 2, cv::LINE_AA);
				}
			}
		}

		//imwrite("./standard_hough.bmp", hough);
	}
	else if (m_pParent->m_houghTransform == 2)
	{
		std::vector<cv::Vec4i> lines;
		HoughLinesP(edge, lines, m_pParent->m_rhoResolution, CV_PI / m_accumulatorWidth, m_pParent->m_houghTreshold, m_pParent->m_houghMinLineLength, m_pParent->m_houghMaxLineGap);
		m_linesFound = lines.size();

		for (size_t i = 0; i < lines.size(); i++)
		{
			cv::Vec4i l = lines[i];

			int yTop = l[1] < l[3] ? l[1] : l[3];

			if (yTop < yMin)
			{
				yMin = yTop;
				closestLine = i;
			}

			if (saveImage)
				line(hough, cv::Point(l[0], l[1]), cv::Point(l[2], l[3]), cv::Scalar(0, 0, 255), 3, CV_AA);
		}

		if (closestLine != -1)
		{
			cv::Vec4i lClose = lines[closestLine];

			double crossTrackError = lClose[1] < lClose[3] ? lClose[0] - edge.cols / 2 : lClose[2] - edge.cols / 2;	//A bad approximation
			double cableAngle = -atan((double)(lClose[2] - lClose[0]) / (lClose[3] - lClose[1]));

			estimateCablePose(crossTrackError, cableAngle, edge.cols / 2);
			sendResult();
		}
		else
		{
			//std::cout << "Did not find any lines" << std::endl;
		}
	}

	m_rowStart++;

	QueryPerformanceCounter(&t4);

	if (saveImage)
	{
		//imwrite("./canny_image.bmp", edge);
		imwrite("./hough.bmp", hough);
		if (m_linesWritten % 10 == 0)
		{
			std::vector<int> compression_params;
			compression_params.push_back(CV_IMWRITE_PNG_COMPRESSION);
			compression_params.push_back(5);

			try {
				imwrite("./HoughImages/hough" + std::to_string(m_linesWritten) + ".png", hough, compression_params);
			}
			catch (std::runtime_error& ex) {
				fprintf(stderr, "Exception converting image to PNG format: %s\n", ex.what());
			}		
		}
	}
	
	filterTime = (t2.QuadPart - t1.QuadPart) * 1000.0 / frequency.QuadPart;
	edgeTime = (t3.QuadPart - t2.QuadPart) * 1000.0 / frequency.QuadPart;
	houghTime = (t4.QuadPart - t3.QuadPart) * 1000.0 / frequency.QuadPart;
	totalTime = (t4.QuadPart - t1.QuadPart) * 1000.0 / frequency.QuadPart;

	//std::cout << "ImageProcessor: found " << linesFound << " lines in " << totalTime << " ms." << std::endl;

	if (m_pParent->m_houghTransform == 0)
		saveData(m_pParent->m_houghTransform, edge.rows, edge.cols, filterTime, edgeTime, houghTime, maxAccumulation, m_pParent->m_cannyMinTres, m_pParent->m_cannyMaxTres, m_pParent->m_houghTreshold, m_accumulatorWidth, edgePoints, cableTres);
	else
		saveData(m_pParent->m_houghTransform, edge.rows, edge.cols, filterTime, edgeTime, houghTime, m_linesFound, m_pParent->m_cannyMinTres, m_pParent->m_cannyMaxTres, m_pParent->m_houghTreshold, m_accumulatorWidth, edgePoints, cableTres);
}

unsigned int ImageProcessor::houghTransform(cv::Mat& edge, unsigned int treshold)
{
	unsigned char* img_data = edge.data;
	unsigned int maxValue = 0;
	int I = -1, K = -1;

	m_accumulatorHeight = (int)round((sqrt(pow(edge.rows, 2) + pow(edge.cols, 2))) / 2) * 2;

	m_originX = (double)edge.cols / 2.0;
	m_originY = (double)edge.rows / 2.0;

	//cv::Mat houghAccumulator2(m_accumulatorHeight, m_accumulatorWidth, CV_16U, cvScalar(255));

	try {
		houghAccumulator = new unsigned int[m_accumulatorHeight * m_accumulatorWidth]();
	}
	catch (std::exception& e)
	{
		std::cout << "houghTransform: Standard exception: " << e.what() << std::endl;
		return 0;
	}

	//Iterate through each point (x,y) in image
	for (int y = 0; y < edge.rows; y++)
	{
		for (int x = 0; x < edge.cols; x++)
		{
			//check if pixel is a part of an edge
			if (img_data[(y*edge.cols) + x] && abs(edge.cols / 2 - x) > edge.cols / 10)
			{
				//calculate hough transform and add weight
				for (int k = 0; k < m_accumulatorWidth; k++)
				{
					double rho = ((double)x - m_originX) *cosTable[k] + ((double)y - m_originY) * sinTable[k];

					if (abs(rho) >= m_accumulatorHeight/2)
						continue;

					//houghAccumulator2.at<short int>((int)round(rho + m_accumulatorHeight / 2), k) -= 4;

					houghAccumulator[(int)floor(rho + m_accumulatorHeight / 2) * m_accumulatorWidth + k]++;
					
					/*
					// quick method to find max accumulation:
					if (++houghAccumulator[(int)round(rho + accumulatorHeight / 2) * accumulatorWidth + k] > maxValue)
					{
						maxValue = houghAccumulator[(int)round(rho + accumulatorHeight / 2) * accumulatorWidth + k];
						K = k;
						I = (int)round(rho + accumulatorHeight / 2);
					}
					*/
				}
			}
		}
	}

	// slow method to find max with accumulator reduction
	for (int i = 0; i < m_accumulatorHeight; i++)
	{
		for (int k = 0; k < m_accumulatorWidth; k++)
		{
			houghAccumulator[i*m_accumulatorWidth + k] = (unsigned short)round(houghAccumulator[i*m_accumulatorWidth + k] * (1 - m_pParent->m_accuReduction*sinTable[k]));

			if (houghAccumulator[i*m_accumulatorWidth + k] > maxValue)
			{
				maxValue = houghAccumulator[i*m_accumulatorWidth + k];
				I = i;
				K = k;
			}
		}
	}
	
	double crossTrackError, cableAngle;

	if (I != -1)
	{
		//double theta = K * CV_PI / accumulatorWidth;	//0 : PI
		double rho = I - m_accumulatorHeight / 2;			//-d/2 : d/2

		double a = cosTable[K], b = sinTable[K];
		double x0 = a*rho, y0 = b*rho;

		crossTrackError = x0*a + (y0 - (double)m_originY)*b;
		cableAngle = K * CV_PI / m_accumulatorWidth;

		if (cableAngle > CV_PI / 2)
		{
			crossTrackError = -crossTrackError;
			cableAngle -= CV_PI;
		}

		if (m_pParent->m_bSaveImages)
		{
			//circle(houghAccumulator2, cv::Point(K, I), 8, 0, 2, 8, 0);
			//imwrite("houghAccumulator.bmp", houghAccumulator2);
		}

		estimateCablePose(crossTrackError, cableAngle, edge.cols / 2);
	}

	if (maxValue >= treshold)
		sendResult();
	
	return maxValue;
}

// get lines from hough transform obtained from http://www.keymolen.com/2013/05/hough-transformation-c-implementation.html by Bruno Keymolen. Only used for plotting
std::vector< std::pair< std::pair<int, int>, std::pair<int, int> > > ImageProcessor::getLines(unsigned int threshold)
{
	std::vector< std::pair< std::pair<int, int>, std::pair<int, int> > > lines;

	if (houghAccumulator == 0)
	{
		return lines;
	}

	for (int i = 0; i < m_accumulatorHeight; i++)
	{
		for (int k = 0; k < m_accumulatorWidth; k++)
		{
			if (houghAccumulator[i*m_accumulatorWidth + k] >= threshold)
			{
				int max = houghAccumulator[i*m_accumulatorWidth + k];
				for (int ly = -4; ly <= 4; ly++)
				{
					for (int lx = -4; lx <= 4; lx++)
					{
						if ((ly + i >= 0 && ly + i<m_accumulatorHeight) && (lx + k >= 0 && lx + k < m_accumulatorWidth))
						{
							if ((int)houghAccumulator[((i + ly)*m_accumulatorWidth) + (k + lx)] > max)
							{
								max = houghAccumulator[((i + ly)*m_accumulatorWidth) + (k + lx)];
								ly = lx = 5;
							}
						}
					}
				}
				if (max >(int)houghAccumulator[(i*m_accumulatorWidth) + k])
					continue;

				int x1, y1, x2, y2;
				x1 = y1 = x2 = y2 = 0;
				int quarterOfPi = (int)round(((float)m_accumulatorWidth) / 4);

				if (k >= quarterOfPi && k <= 3 * quarterOfPi)
				{
					x1 = (int) 0;
					y1 = (int)(((double)(i - m_accumulatorHeight / 2) - (x1 - m_originX) * cos(k*CV_PI / m_accumulatorWidth)) / sin(k*CV_PI / m_accumulatorWidth) + m_originY);
					x2 = (int)m_originX * 2;
					y2 = (int)(((double)(i - m_accumulatorHeight / 2) - (x2 - m_originX) * cos(k*CV_PI / m_accumulatorWidth)) / sin(k*CV_PI / m_accumulatorWidth) + m_originY);
				}
				else
				{
					y1 = (int) 0;
					x1 = (int)(((double)(i - m_accumulatorHeight / 2) - (y1 - m_originY) * sin(k*CV_PI / m_accumulatorWidth)) / cos(k*CV_PI / m_accumulatorWidth) + m_originX);
					y2 = (int)m_originY * 2;
					x2 = (int)(((double)(i - m_accumulatorHeight / 2) - (y2 - m_originY) * sin(k*CV_PI / m_accumulatorWidth)) / cos(k*CV_PI / m_accumulatorWidth) + m_originX);
				}

				lines.push_back(std::pair< std::pair<int, int>, std::pair<int, int> >(std::pair<int, int>(x1, y1), std::pair<int, int>(x2, y2)));
			}
		}
	}

	//std::cout << "lines: " << lines.size() << " " << threshold << std::endl;

	delete(houghAccumulator);
	return lines;
}

void ImageProcessor::applyTVG(cv::Mat& image)
{
	//float altitude = m_pParent->getSideScanThread()->ssData.altitude;
	float vSound = m_pParent->getSideScanThread()->ssData.soundSpeed;
	float secPerLine = m_pParent->getSideScanThread()->ssData.secPerLine1;

	//double tPing = 50.0 / 750.0;
	double maxSlantRange = secPerLine*vSound / 2.0;  // 50.0 m / 750.0 m/s = 
	double a = m_pParent->m_TVGMultiplier;

	for (int i = 0; i < image.cols; i++)
	{
		double range;

		//calculate range dependent on channel
		if (i < (image.cols / 2))
			range = (image.cols / 2 - i) * maxSlantRange / (image.cols / 2 - 1);
		else
			range = (i - image.cols / 2) * maxSlantRange / (image.cols / 2 - 1);

		double gain = a*range;                   //40 * log10(range) + 2 * alpha * range;

		for (int j = 0; j < image.rows; j++)
		{
			image.at<double>(j, i) = image.at<double>(j, i) + gain / 128.0;
		}
	}
}

void ImageProcessor::estimateCablePose(double pixelCrossTrackError, double relativePixAngle, int ssArrayLength)
{
	double headingAUV = m_pParent->getNavThread()->posDataStruct.m_dHeading;

	if (m_scaleImage)
	{
		m_cableHeading = relativePixAngle + headingAUV;
		m_crossTrackError = pixelCrossTrackError * m_meterPerPixel;
		return;
	}

	// measurements in SI units
	float altitude = m_pParent->getSideScanThread()->ssData.altitude;
	float vSound = m_pParent->getSideScanThread()->ssData.soundSpeed;
	float speed = m_pParent->getSideScanThread()->ssData.speed / ((float) 1.852);
	float secPerLine = m_pParent->getSideScanThread()->ssData.secPerLine1;

	double maxSlantRange = secPerLine*vSound / 2.0;
	//double timeBetweenPings = 899.7 / 7041.0;

	//along-track and orthogonal distance in pixels
	double yPix = pixelCrossTrackError*sin(-relativePixAngle);
	double xPix = pixelCrossTrackError*cos(relativePixAngle);

	double range = xPix / ssArrayLength * maxSlantRange;
	int signX = xPix < 0 ? -1 : 1;
	int signCrossTrack = pixelCrossTrackError < 0 ? -1 : 1;

	//along-track and orthogonal distance in meters
	double y, x;

	if (abs(range) > altitude)
		x = signX*sqrt(pow(range, 2) - pow(altitude, 2));
	else
		x = 0;

	y = yPix * 2 * secPerLine * speed;

	// alternative heading estimate
	double relativeHeading;

	if (x > 0)
	{
		relativeHeading = -atan2(y, x);
	}
	else if (x < 0)
	{
		relativeHeading = atan2(y, -x);
	}
	else if (x == 0)
	{
		relativeHeading = relativePixAngle;
	}

	m_crossTrackError = signCrossTrack*sqrt(pow(x, 2) + pow(y, 2));
	m_cableHeading = relativePixAngle + headingAUV;

	//double headingCable2 = headingAUV + relativeHeading;

	std::cout << "ImageProcessor: x_hat = " << m_crossTrackError << "  " << relativePixAngle * 180 / 3.1415 << std::endl;//"   auv heading = " << headingAUV * 180 / 3.1415 << std::endl;
}

void ImageProcessor::sendResult()
{
	// check if angular rate is too high
	if (abs(m_pParent->getNavThread()->posDataStruct.m_dZAngRateB) > m_pParent->m_maxAngRate)
	{
		std::cout << "ImageProcessor: too high angular rate: " << m_pParent->getNavThread()->posDataStruct.m_dZAngRateB << "  max: " << m_pParent->m_maxAngRate << std::endl;
		return;
	}
	
	double dxReflection;
	double depth = m_pParent->getNavThread()->posDataStruct.m_dDepth;
	double altitude = m_pParent->getSideScanThread()->ssData.altitude;

	if (abs(depth) > abs(altitude))
	{
		dxReflection = sqrt(pow(depth, 2) - pow(altitude, 2));
	}
	else
	{
		dxReflection = 0;
	}

	if (dxReflection < 8)
		return;

	char buf[100];
	char* pBuf = buf;

	char messageType = 'n';
	m_messageNumber++;

	memcpy(pBuf, &messageType, sizeof(messageType));
	pBuf += sizeof(messageType);

	memcpy(pBuf, &m_messageNumber, sizeof(m_messageNumber));
	pBuf += sizeof(m_messageNumber);

	memcpy(pBuf, &m_crossTrackError, sizeof(m_crossTrackError));
	pBuf += sizeof(m_crossTrackError);

	memcpy(pBuf, &m_cableHeading, sizeof(m_cableHeading));
	pBuf += sizeof(m_cableHeading);

	m_pParent->getSideScanThread()->sendMessage(buf, sizeof(buf));

	//std::cout << m_messageNumber << std::endl;

	//std::cout << "ImageProcessor: x_hat = " << m_crossTrackError << "  " << m_cableHeading * 180 / 3.1415 << std::endl;//"   auv heading = " << m_cableHeading * 180 / 3.1415 << std::endl;
}

void ImageProcessor::scaleImage(cv::Mat& image)
{
	cv::Mat image2;
	//float altitude = m_pParent->getSideScanThread()->ssData.altitude;
	//float vSound = m_pParent->getSideScanThread()->ssData.soundSpeed;
	float speed = m_pParent->getSideScanThread()->ssData.speed / ((float) 1.852);
	float secPerLine = m_pParent->getSideScanThread()->ssData.secPerLine1;

	double fx = 50.0 / (image.cols / 2.0) / m_meterPerPixel;
	double fy = 2 * secPerLine*speed / m_meterPerPixel;

	if (round(fy*image.rows) < 1 || round(fx*image.cols) < 1)
		return;

	cv::Size dsize = cv::Size(round(fx*image.cols), round(fy*image.rows));
	resize(image, image, dsize, fx, fy, cv::INTER_LINEAR);

	if (m_pParent->m_bSaveImages)
		cv::imwrite("./scaled.bmp", image);
}

void ImageProcessor::saveData(int ht, int imgHeight, int imgWidth, double filtT, double edgeT, double houghT, unsigned int res, int cannyMin, int cannyMax, int houghTres, int rhoRes, int edgePoints, int cableTres)
{
	//std::cout << "ImageProcessor: houghT = " << std::setprecision(4) << houghT << "  maxAcc/lines = " << res << "  lines written = " << m_linesWritten << std::endl;

	std::string folder = m_pParent->getSideScanThread()->getOutputFolder();
	std::string timeStart = m_pParent->getSideScanThread()->getTimeStart();
	std::string name = "ImageProcessing";

	if (folder.empty())
		return;

	std::ofstream saveFile;
	saveFile.open(folder + name + timeStart + ".txt", std::ios_base::app);
	
	saveFile << ht << " ";
	saveFile << m_linesWritten << " ";
	saveFile << imgHeight << " ";
	saveFile << imgWidth << " ";
	saveFile << filtT << " ";
	saveFile << edgeT << " ";
	saveFile << houghT << " ";
	saveFile << res << " ";
	saveFile << cannyMin << " ";
	saveFile << cannyMax << " ";
	saveFile << houghTres << " ";
	saveFile << rhoRes << " ";
	saveFile << edgePoints << " ";
	saveFile << cableTres << " ";
	saveFile << m_crossTrackError << " ";
	saveFile << m_cableHeading << " ";
	saveFile << std::setprecision(15) << m_pParent->getNavThread()->posDataStruct.m_dLatitude << " ";
	saveFile << std::setprecision(15) << m_pParent->getNavThread()->posDataStruct.m_dLongitude << " ";
	saveFile << m_pParent->getNavThread()->posDataStruct.m_dHeading << " ";

	saveFile << "\n";
	saveFile.close();
}