#pragma once

#ifndef WIN32_LEAN_AND_MEAN			//prevents windows.h from including winsock.h
#define WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <limits>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include "opencv2/imgproc/imgproc.hpp"

#define ARRAYS_PER_IMAGE 80
#define SS_ARRAY_LENGTH 1182 // 256  //

class CCableDetector;

class ImageProcessor
{
public:
	ImageProcessor(CCableDetector *pParent);
	virtual ~ImageProcessor();

	DWORD main();
	HANDLE imgHandle;
	bool readData();
	void saveData(int ht, int imgHeight, int imgWidth, double filtT, double edgeT, double houghT, unsigned int res, int cannyMin, int cannyMax, int houghTres, int rhoRes, int edgePoints, int cableTres);
	void lineDetection();
	bool startThread();
private:
	unsigned int houghTransform(cv::Mat& edge, unsigned int treshold);
	std::vector< std::pair< std::pair<int, int>, std::pair<int, int> > > getLines(unsigned int threshold);
	void applyTVG(cv::Mat& image);
	void estimateCablePose(double pixelCrossTrackError, double relativePixAngle, int ssArrayLength);
	void sendResult();
	void scaleImage(cv::Mat& image);

	CCableDetector *m_pParent;
	bool m_bRunning;

	cv::Mat src;
	std::ifstream& gotoLine(std::ifstream& file, unsigned int num);
	int m_startLine;
	int m_rowStart;
	int m_linesWritten;
	double m_cableHeading;
	double m_crossTrackError;
	bool m_scaleImage;
	double m_meterPerPixel;

	//double data[ARRAYS_PER_IMAGE][SS_ARRAY_LENGTH * 2];
	double *data1;

	int m_accumulatorHeight;	//radius
	int m_accumulatorWidth;	//angle
	double m_originX;
	double m_originY;
	unsigned int * houghAccumulator;
	double * sinTable;
	double * cosTable;

	int m_linesFound;
	int m_messageNumber;
};