
#include "SideScanThread.h"
#include "CableDetector.h"
#include <iostream>
#include <fstream>

#include <sstream>
#include <vector>
#include <iterator>

#define BUFLEN 20000  //Max length of buffer
int iterator = 10;

CSideScanThread::CSideScanThread(CCableDetector *pParent) :
	m_pParent(pParent),
	m_linesWritten(0),
	m_ssArrayLength(0),
	m_outputFolder(""),
	m_timeStart("")
{
	//std::cout << "SideScanThread: Size of header: " << sizeof(SideScanDataHeader2) << std::endl;
}

CSideScanThread::~CSideScanThread()
{
	CloseHandle(ssHandle);
	closesocket(m_socket);
	WSACleanup();
}

static DWORD WINAPI StaticThreadStart(void* Param)
{
	CSideScanThread* This = (CSideScanThread*)Param;
	return This->main();
}

unsigned int __stdcall ssThreadStart(void* data)
{
	//printf("SideScanThread: _beginthreadex thread id: %d \n", GetCurrentThreadId());
	CSideScanThread* This = (CSideScanThread*)data;
	return This->main();
}

DWORD CSideScanThread::main() {
	
	int slen = sizeof(si_other);
	char buf[BUFLEN];

	std::cout << "SideScanThread running." << std::endl;
	while (true) //(m_linesWritten < 900)
	{
		Sleep(10);

		char message[100];
		char* pMessage = message;
		char messageType = 'r';

		memcpy(pMessage, &messageType, sizeof(messageType));
		pMessage += sizeof(messageType);

		//send message to ensure connection
		if (sendto(m_socket, message, sizeof(message), 0, (struct sockaddr *) &si_other, slen) == SOCKET_ERROR)
		{
			printf("SideScanThread: sendto() failed with error code : %d\n", WSAGetLastError());
		}

		//clear the buffer by filling null, it might have previously received data
		memset(buf, '\0', BUFLEN);

		//try to receive some data, this is not a blocking call
		if (recvfrom(m_socket, buf, BUFLEN, 0, (struct sockaddr *) &si_other, &slen) == SOCKET_ERROR)
		{
			if (iterator == 20) {
				//printf("SideScanThread: recvfrom() failed with error code : %d\n", WSAGetLastError());
				iterator = 0;
			}
			iterator++;
		}
		else {
			saveBuf(buf);
		}
	}

	return 1;
}

void CSideScanThread::sendMessage(char* buf, int bufSize)
{
	int slen = sizeof(si_other);

	if (sendto(m_socket, buf, bufSize, 0, (struct sockaddr *) &si_other, slen) == SOCKET_ERROR)
	{
		printf("SideScanThread: sendto() failed with error code : %d\n", WSAGetLastError());
	}
}

bool CSideScanThread::saveBuf(char* buf)
{
	long long timeStamp;
	unsigned int ping;
	float speed, altitude, soundSpeed, secPerLine1, secPerLine2;
	int ssArrayLength;

	char* pbuf = buf;

	timeStamp = *reinterpret_cast<long long*>(buf);
	pbuf += sizeof(timeStamp);

	memcpy(&ping, pbuf, sizeof(ping));
	pbuf += sizeof(ping);

	memcpy(&speed, pbuf, sizeof(speed));
	pbuf += sizeof(speed);

	memcpy(&altitude, pbuf, sizeof(altitude));
	pbuf += sizeof(altitude);

	memcpy(&soundSpeed, pbuf, sizeof(soundSpeed));
	pbuf += sizeof(soundSpeed);

	memcpy(&secPerLine1, pbuf, sizeof(secPerLine1));
	pbuf += sizeof(secPerLine1);

	memcpy(&secPerLine2, pbuf, sizeof(secPerLine2));
	pbuf += sizeof(secPerLine2);

	memcpy(&ssArrayLength, pbuf, sizeof(ssArrayLength));
	pbuf += sizeof(ssArrayLength);

	std::vector<double> ss_data(2 * ssArrayLength);
	memcpy(ss_data.data(), pbuf, 2 * ssArrayLength * sizeof(double));

	//std::cout << "SideScanThread: ping: " << ping << "  ssData: " << ss_data.front() << " " << ss_data.back() << std::endl;

	if (ssArrayLength < 1)
		return false;
	else
		m_ssArrayLength = ssArrayLength;

	std::ofstream saveFile;
	saveFile.open(m_outputFolder + m_outputFile, std::ios_base::app);
	
	saveFile << timeStamp << " ";
	saveFile << ping << " ";
	saveFile << speed << " ";
	saveFile << altitude << " ";
	saveFile << soundSpeed << " ";
	saveFile << secPerLine1 << " ";
	saveFile << secPerLine2 << " ";
	saveFile << ssArrayLength << " ";

	for (int i = 0; i < 2 * ssArrayLength; i++) {
		saveFile << ss_data[i] << " ";
	}

	saveFile << "\n";
	saveFile.close();

	ssData.speed = speed;
	ssData.altitude = altitude;
	ssData.soundSpeed = soundSpeed;
	ssData.secPerLine1 = secPerLine1;
	ssData.secPerLine2 = secPerLine2;

	m_linesWritten++;

	return true;
}

bool CSideScanThread::startThread()
{
	// Check if module has been initialized, if not, initialize module now
	if (!initializeWinsock())
	{
		WSACleanup();
		// Log the event to the event log
		//m_pParent->storeEventString("CSideScanThread: Failed to initialize module.");
		std::cout << "CableDetector::SideScanThread::initializeWinsock() failed. Retrying..." << std::endl;
		Sleep(1000);
		return false;
	}

	//DWORD myThreadID;
	//ssHandle = CreateThread(0, 0, StaticThreadStart, (void*) this, 0, &myThreadID);
	ssHandle = (HANDLE)_beginthreadex(0, 0, &ssThreadStart, (void*) this, 0, 0);

	m_bRunning = true;

	SYSTEMTIME time;
	GetLocalTime(&time);

	char cTime[7];
	sprintf_s(cTime, 7, "%02d%02d%02d", (int) time.wHour, (int) time.wMinute, (int) time.wSecond);

	m_timeStart = cTime;

	m_outputFolder = "CableDetectorData/";
	m_outputFile = "SideScanData" + m_timeStart + ".txt";

	if (CreateDirectoryA(m_outputFolder.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError())
	{
		//std::cout << "CableDetector: writing results to " << OutputFolder << std::endl;
	}
	else
	{
		std::cout << "CableDetector::SideScanThread: Failed to create directory " << m_outputFolder << std::endl;
	}

	return m_bRunning;
}

bool CSideScanThread::initializeWinsock()
{
	WSADATA wsa;

	//Initialise winsock
	printf("SideScanThread: Initialising Winsock...\n");
	if (WSAStartup(MAKEWORD(2, 2), &wsa) != 0)
	{
		printf("SideScanThread: Failed. Error Code : %d\n", WSAGetLastError());
		return false;
	}
	printf("SideScanThread: Initialised.\n");

	//create socket
	if ((m_socket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == SOCKET_ERROR)
	{
		printf("SideScanThread: socket() failed with error code : %d\n", WSAGetLastError());
		return false;
	}

	//setup address structure
	memset((char *)&si_other, 0, sizeof(si_other));
	si_other.sin_family = AF_INET;
	si_other.sin_port = htons(m_pParent->getSideScanPort());
	si_other.sin_addr.S_un.S_addr = inet_addr(m_pParent->getCableTrackerIP());

	//For nonblocking udp receive function
	//u_long iMode = 1;
	//ioctlsocket(m_socket, FIONBIO, &iMode);

	return true;
}