/*
	CtdThread.cpp

	Copyright: 		Department of Marine Technology, NTNU
	Author: 		Petter Norgren
	Date created: 	2016.09.09
*/


#include "CtdThread.h"
#include "RemusReconImpl.h"

#include "PayloadInterface/PPDataBase.h"

CtdThread::CtdThread(CRemusReconImpl *pParent)
	: OAL::CThread("CtdThread")
	, m_mutex("CTDmutex")
	, m_pParent(pParent)
	, m_bInitialized(false)
	, m_bRunning(false)
{
	// Empty constructor
}

CtdThread::~CtdThread()
{
	// Clean-up
}

void CtdThread::main()
{
	CCTDData data;

	while( !m_ShutDown.wait(0) )
	{	
		//std::cout << "CtdThread" << std::endl;
		if(CPPDataBase::getInstance().wait(CPPDataBase::eCTDData, 100))
		{
			memset(&data, 0, sizeof(CCTDData));
			CPPDataBase::getInstance().read(CPPDataBase::eCTDData, data);

			// Set member data (critical section since we have both readers and writers)
			setData(data);
		}

		// Sleep for 10ms
		wait(10);
	}
}

bool CtdThread::initialize()
{

	// Check that module is not already initialized
	if(m_bInitialized)
	{
		return true;
	}
	
	try
	{
		// Log an event to the event log
		m_pParent->storeEventString("CtdThread: Initialization complete!");
		m_bInitialized = true;
		return true;
	}
	catch ( ... )
	{
		m_pParent->storeEventString("CtdThread: Exception when initializing.");
		return false;
	}
}

bool CtdThread::startThread()
{
	try
	{
		if(isRunning())
		{
			// Log the event to the event log
			m_pParent->storeEventString("CtdThread: Thread already running. Disregarding start thread command.");
			return true;
		}

		if(!initialize())
		{
			// Log the event to the event log
			m_pParent->storeEventString("CtdThread: Failed to initialize module.");
			return false;
		}

		// Start thread
		start();

		m_bRunning = true;

		// Log an event to the event log
		m_pParent->storeEventString("CtdThread: Thread started.");
		return true;
	}
	catch ( ... )
	{
		m_pParent->storeEventString("CtdThread: Exception when trying to start thread.");
		return false;
	}
}

void CtdThread::stopThread()
{
	// Shutdown thread
	if(isRunning()){
		(void) shutDown();

		// Log an event to the event log
		m_pParent->storeEventString("CtdThread: Thread stopped.");

		m_bRunning = false;
	}
}

void CtdThread::setData(const CCTDData &data)
{
	OAL::CCriticalSectionLock Lock(m_mutex);

	m_data = data;
}

void CtdThread::getData(CCTDData &data)
{
	OAL::CCriticalSectionLock Lock(m_mutex);

	data = m_data;
}