#include "DebugThread.h"
#include "RemusReconImpl.h"

#include "PayloadInterface/PPDataBase.h"

// Include necessary sensor databases to be able to debug plugin
#include "CommonCode/Storable Sensor Data/NavSolutionData.h"
#include "CommonCode/Storable sensor data/SoundVelocityData.h"
#include "CommonCode/RTOS abstraction layer/config.h"

#include <iostream>

CDebugThread::CDebugThread(CRemusReconImpl *pParent): 
m_bInitialized(false),
m_bRunning(false),
OAL::CThread("DebugThread"),
m_pParent(pParent),
bmpWidth(0),
bmpHeight(0),
m_navNumber(0),
m_pingNumber(0),
m_ctdNumber(0),
m_startTime(0)
{
	//
}

CDebugThread::~CDebugThread()
{
	//if (m_pComPort != NULL)
	//{
	//	m_pComPort->close();
	//	delete m_pComPort;
	//	m_pComPort = NULL;
	//}
}

void CDebugThread::main()
{
	std::cout << "DebugThread: Running.." << std::endl;
	CNavSolutionData posData;
	CSoundVelocityData svData;
	CSideScanData ssData;
	CCTDData ctdData;

	memset(&posData, 0, sizeof(CNavSolutionData));
	posData.m_dDepth = 5.203; 
	posData.m_dHeading = 0; //0.785398163;
	posData.m_dLongitude = 10.686951;
	posData.m_dLatitude = 59.905037;
	posData.m_dPitch = 0.034906585;
	posData.m_dRoll = -0.104719755;
	posData.m_dDownVelocityL = 1.04;

	memset(&ssData, 0, sizeof(CSideScanData));
	ssData.m_PingHeader.Body.PingNumber = 0;
	std::vector<double> sss[2];
	//sss[0].resize(256,11.23);
	//sss[1].resize(256,22.34);
	int sssSize = sidescanData[0].size()-5;
	sss[0].resize(sssSize/2,0);
	sss[1].resize(sssSize/2,0);
	unsigned char* bmpData = readBMP("./cable2.bmp");
	//sss[0].resize(bmpWidth/2,0);
	//sss[1].resize(bmpWidth/2,0);
	int currentRow = 0;

	ssData.m_data[0] = sss[0];
	ssData.m_data[1] = sss[1];
	ssData.m_PingHeader.Body.Speed = 1.5;
	ssData.m_PingHeader.Body.Altitude = 5.5;
	ssData.m_PingHeader.Body.PingNumber = (int) sidescanData[m_pingNumber][1];
	//double ssReading = 0.0;

	memset(&ctdData, 0, sizeof(CCTDData));

	memset(&svData, 0, sizeof(CSoundVelocityData));
	svData.m_dSoundVelocity = 1488.7;

	CAbsTime simStartTime = CHPTimer::getInstance()->getAbsTime();
	std::cout << "ss pingnum: " << m_pingNumber << "   navnum: "<< m_navNumber<< std::endl;

	while( !m_ShutDown.wait(0))
	{
		sleep(50);

		if ( m_pingNumber >= (int) sidescanData.size())
		{
			std::cout << "Debug: End of sidescan reached" << std::endl;
			break;
		}
		if ((sidescanData[m_pingNumber][0]*1000 - m_startTime)  < (CHPTimer::getInstance()->getAbsTime() - simStartTime))
		{
			//std::cout << "ss time: " << sidescanData[m_pingNumber][0]*1000 << "    nav time: "<< navData[m_navNumber][0]<< std::endl;
			for(int i = 0; i< (int) sidescanData[0].size();i++)
				if(i == 0)
					continue;
				else if (i == 1)
					ssData.m_PingHeader.Body.Speed = (float) sidescanData[m_pingNumber][i];
				else if (i == 2)
					ssData.m_PingHeader.Body.Altitude = (float) sidescanData[m_pingNumber][i];
				else if (i == 3)
					ssData.m_PingHeader.Body.SoundVelocity = (float) sidescanData[m_pingNumber][i]*2; //sound velocity round trip = 750
				else if (i == 4)
					ssData.m_PingChanHeader[0].SecondsPerLine = (float) sidescanData[m_pingNumber][i];
				else if (i < (int)(5+sssSize/2))
					sss[0][i-5] = sidescanData[m_pingNumber][i];
				else
					sss[1][i-5-sssSize/2] = sidescanData[m_pingNumber][i];

				ssData.m_data[0] = sss[0];
				ssData.m_data[1] = sss[1];
				ssData.m_nTimeStamp = CHPTimer::getInstance()->getAbsTime();
				ssData.m_PingHeader.Body.PingNumber++;
				CPPDataBase::getInstance().write( CPPDataBase::eSideScanData, ssData);

				//if (m_pingNumber < sss.size())
				m_pingNumber++;
		}

		if ((navData[m_navNumber][0]-m_startTime) <  (CHPTimer::getInstance()->getAbsTime() - simStartTime))
		{
			posData.m_nTimeStamp = CHPTimer::getInstance()->getAbsTime();
			posData.m_dLatitude = navData[m_navNumber][1];
			posData.m_dLongitude = navData[m_navNumber][2];
			posData.m_dDepth = navData[m_navNumber][3];
			posData.m_dRoll = navData[m_navNumber][4];
			posData.m_dPitch = navData[m_navNumber][5];
			posData.m_dHeading = navData[m_navNumber][6];
			posData.m_dXVelocityB = navData[m_navNumber][7];
			posData.m_dYVelocityB = navData[m_navNumber][8];
			posData.m_dZVelocityB = navData[m_navNumber][9];
			posData.m_dNorthVelocityL = navData[m_navNumber][10];
			posData.m_dEastVelocityL = navData[m_navNumber][11];
			posData.m_dDownVelocityL = navData[m_navNumber][12];
			posData.m_dXAngRateB = navData[m_navNumber][13];
			posData.m_dYAngRateB = navData[m_navNumber][14];
			posData.m_dZAngRateB = navData[m_navNumber][15];
			posData.m_dXAccB = navData[m_navNumber][16];
			posData.m_dYAccB = navData[m_navNumber][17];
			posData.m_dZAccB = navData[m_navNumber][18];
			posData.m_dLatitudeStdDev = navData[m_navNumber][19];
			posData.m_dLongitudeStdDev = navData[m_navNumber][20];
			posData.m_dLatLongCoVariance = navData[m_navNumber][21];
			posData.m_dDepthStdDev = navData[m_navNumber][22];
			posData.m_dRollStdDev = navData[m_navNumber][23];
			posData.m_dPitchStdDev = navData[m_navNumber][24];
			posData.m_dHeadingStdDev = navData[m_navNumber][25];

			ReconStatus_t reconState;
			//m_pParent->getReconDriver()->getRemoteControlState(reconState);
			
			reconState.Altitude = ssData.m_PingHeader.Body.Altitude;
			reconState.Latitude = navData[m_navNumber][1]*180/PI;
			reconState.Longitude = navData[m_navNumber][2]*180/PI;
			reconState.Depth = navData[m_navNumber][3];
			reconState.Pitch = navData[m_navNumber][5];
			reconState.Roll = navData[m_navNumber][4];
			reconState.Speed = navData[m_navNumber][7];
			reconState.Heading = navData[m_navNumber][6];

			m_pParent->getReconDriver()->setRemoteControlState(reconState);
			CPPDataBase::getInstance().write( CPPDataBase::ePositionData, posData);

			//if (m_navNumber < navData.size())
			m_navNumber++;
		}
		
		if ((ctdDataMatrix[m_ctdNumber][0]-m_startTime) <  (CHPTimer::getInstance()->getAbsTime() - simStartTime))
		{
			ctdData.m_nTimeStamp = CHPTimer::getInstance()->getAbsTime();
			ctdData.m_dConductivity = ctdDataMatrix[m_ctdNumber][1];
			ctdData.m_dTemperature = ctdDataMatrix[m_ctdNumber][2];
			ctdData.m_dPressure = ctdDataMatrix[m_ctdNumber][3];
			ctdData.m_dSoundVelocity = ctdDataMatrix[m_ctdNumber][4];

			CPPDataBase::getInstance().write( CPPDataBase::eCTDData, ctdData);
			m_ctdNumber++;
		}

		if (false)
		{
			m_pingNumber++;

			// Write nav-data
			posData.m_nTimeStamp = CHPTimer::getInstance()->getAbsTime();
			CPPDataBase::getInstance().write( CPPDataBase::ePositionData, posData);

			// Write soundvelocity-data
			svData.m_nTimeStamp = CHPTimer::getInstance()->getAbsTime();
			CPPDataBase::getInstance().write( CPPDataBase::eSoundVelocityData, svData);

			// Write sidescan data

			int j = 0;
			for(int i = bmpWidth*currentRow*3; i < (bmpWidth*(currentRow)+ bmpWidth/2)*3; i += 3)
			{
				sss[0][j] = (int) bmpData[i];
				j++;
			}

			j = 0;
			for(int i = (bmpWidth*currentRow + bmpWidth/2)*3; i < (bmpWidth*(currentRow)+ bmpWidth)*3; i += 3)
			{
				sss[1][j] = (int) bmpData[i];
				j++;
			}

			if (currentRow < bmpHeight-1)
				currentRow++;
			else
				currentRow = 0;

			ssData.m_data[0] = sss[0];
			ssData.m_data[1] = sss[1];
			ssData.m_nTimeStamp = CHPTimer::getInstance()->getAbsTime();
			ssData.m_PingHeader.Body.PingNumber++;
			CPPDataBase::getInstance().write( CPPDataBase::eSideScanData, ssData);
		}
	}

	delete (bmpData);
}

unsigned char* CDebugThread::readBMP(char* filename)
{
	int i;
	FILE* f = fopen(filename, "rb");
	unsigned char info[54];
	fread(info, sizeof(unsigned char), 54, f); // read the 54-byte header

	// extract image height and width from header
	bmpWidth = *(int*)&info[18];
	bmpHeight = *(int*)&info[22];

	std::cout << "DebugThread: image " << filename << " is " << bmpWidth << "x" << bmpHeight << std::endl;

	int size = 3 * bmpWidth * bmpHeight;
	unsigned char* data = new unsigned char[size]; // allocate 3 bytes per pixel
	fread(data, sizeof(unsigned char), size, f); // read the rest of the data at once
	fclose(f);

	for(i = 0; i < size; i += 3)
	{
		unsigned char tmp = data[i];
		data[i] = data[i+2];
		data[i+2] = tmp;
	}

	return data;
}

std::vector<std::vector<double> > CDebugThread::ReadMatrix(const char *filename, int columns)
{
	std::vector<std::vector<double> > table;
	std::ifstream ifs(filename);

	if (!ifs) {
		std::cout << "DebugThread: ERROR! Cannot open file " << filename << std::endl;
		return table;
	}

	if (ifs.is_open())
	{
		int columnNum = 0;
		int rowNum = 0;
		double value;

		std::vector<double> row(columns);
		while (ifs >> value)
		{
			row[columnNum] = value;

			if (columnNum == columns-1)
			{
				columnNum = 0;
				rowNum++;

				table.push_back(row);
			}
			else
				columnNum++;
		}
	}

	ifs.close();

	return table;
}

bool CDebugThread::initialize()
{
	// Check that module is not already initialized
	if(m_bInitialized)
	{
		return true;
	}

	sidescanData = ReadMatrix("./sidescanData3.txt",1029);
	navData = ReadMatrix("./navData.txt",31);
	ctdDataMatrix = ReadMatrix("./ctdData.txt",7);

	CAbsTime simStartTime = CHPTimer::getInstance()->getAbsTime();

	m_startTime = sidescanData[0][0]*1000;

	while ((navData[m_navNumber][0]-m_startTime) <  (CHPTimer::getInstance()->getAbsTime() - simStartTime))
		m_navNumber++;

	while ((sidescanData[m_pingNumber][0]*1000 - m_startTime)  < (CHPTimer::getInstance()->getAbsTime() - simStartTime))
		m_pingNumber++;

	while ((ctdDataMatrix[m_ctdNumber][0] - m_startTime)  < (CHPTimer::getInstance()->getAbsTime() - simStartTime))
		m_ctdNumber++;

	ReconStatus_t reconState;
	m_pParent->getReconDriver()->getRemoteControlState(reconState);

	reconState.Latitude = navData[m_navNumber][1]*180/PI;
	reconState.Longitude = navData[m_navNumber][2]*180/PI;
	reconState.Depth = navData[m_navNumber][3];
	reconState.Pitch = navData[m_navNumber][5];
	reconState.Roll = navData[m_navNumber][4];
	reconState.Speed = navData[m_navNumber][7];
	reconState.Heading = navData[m_navNumber][6];

	m_pParent->getReconDriver()->setRemoteControlState(reconState);

	m_bInitialized = true;

	return m_bInitialized;
}

bool CDebugThread::startThread()
{
	// Check if module has been initialized, if not, initialize module now
	if(!initialize())
	{
		// Log the event to the event log
		m_pParent->storeEventString("DebugThread: Failed to initialize module.");
		std::cout << "ERROR: CableTrackerPlugin: DebugThread::startThread() Unable to initialize thread" << std::endl;
		return false;
	}

	try
	{
		// Start thread
		start();

		m_bRunning = true;

		// Log an event to the event log
		m_pParent->storeEventString("DebugThread: Thread started.");
		return true;
	}
	catch ( ... )
	{
		m_pParent->storeEventString("DebugThread: Exception when trying to start thread.");
		return false;
	}
}

void CDebugThread::stopThread()
{
	// Log an event to the event log
	m_pParent->storeEventString("DebugThread: Thread stopped.");

	// Shutdown thread
	(void) shutDown();

	m_bRunning = false;
}