/*
	GuidanceImpl.h

	This file is for implementation of custom guidance algorithms for the REMUS 100 AUV.
	Make changes to this header file and corresponding source file only!

	See GuidanceInterface for overridable getters (used by GuidanceThread).

	Copyright: 		Department of Marine Technology, NTNU
	Author: 		Petter Norgren
	Date created: 	2016.09.09
*/
#pragma once

#include "GuidanceInterface.h"
#include <vector>   //for std::vector

// Change thread based on what payload data that guidance algorithm uses
#include "CtdThread.h"
//#include "SensorFusion.h"

class CRemusReconImpl;

class CGuidanceImpl : public CGuidanceInterface
{
public:
	CGuidanceImpl(CRemusReconImpl *pParent);
	~CGuidanceImpl();

	double getHeadingCommandRadians() { return m_headingCommandRadians; }
	double getSpeedCommandMeterSec() { return m_speedCommandMeterSec; }
	double getAltitudeCommandMeters() { return m_altitudeCommandMeters; }
	double getDepthCommandMeters() {return m_depthCommandMeters; }
	double getLatitudeCommandRadians() { return m_latitudeCommandRadians; }
	double getLongitudeCommandRadians() { return m_longitudeCommandRadians; }
	bool getWaypointsEnable() { return m_waypointsEnable; }
	bool isMissionComplete() { return m_bMissionComplete; }
	int getCurrentWaypoint() { return m_currentWaypoint; }
	unsigned int getWpLOSNumber() { return m_wpLOSnumber; }
	double getWpCrossTrackError() { return m_crossTrackErrorWP; }
	double getWpCableHeading() { return m_headingWP; }
	bool computeNewCommands(const ReconStatus_t &reconState);
	CtdThread * getCTDThread() { return m_ctdThread; }

private:
	CRemusReconImpl *m_pParent;

	// Change if necessary
	bool m_bInitialized;
	double m_headingCommandRadians;
	double m_speedCommandMeterSec;
	double m_altitudeCommandMeters;
	double m_depthCommandMeters;
	double m_latitudeCommandRadians;
	double m_longitudeCommandRadians;
	bool m_waypointsEnable;
	int m_currentWaypoint;
	std::vector<double> m_waypointsLatitudeRadians;
	std::vector<double> m_waypointsLongitudeRadians;
	int m_startWaypoint;
	int m_stopWaypoint;
	int m_numWaypoints;
	int m_direction;
	int m_directionControl;
	double m_errorIntegral;
	double m_lookaheadDistance;
	double m_integralGain;
	double m_saturationLimit;
	double m_northOffsetPerAltitude;
	double m_eastOffsetPerAltitude;
	double m_crossTrackErrorWP;
	double m_headingWP;
	bool m_bMissionComplete;
    unsigned int m_wpLOSnumber;
	CAbsTime m_prevSampleTime;

	bool useWaypointsOrHeading();
	bool CGuidanceImpl::writeData(char* fileName, double data);
	void initialize();
	bool obtainWaypoints(std::vector<double>& wpLat, std::vector<double>& wpLong, const char* const waypointFile);
	bool readSosiFile(const char* FileName, std::vector<double>& Easting,std::vector<double>& Northing, int curveNumber);
	bool readCoordinates(const char* FileName, std::vector<double>& North, std::vector<double>& East);
	void waypointLOS(const ReconStatus_t &reconState);

	CtdThread *m_ctdThread;
	//CSensorFusion *m_sensorFusion;
};