/*
	GuidanceThread.h

	Copyright: 		Department of Marine Technology, NTNU
	Author: 		Petter Norgren
	Date created: 	2016.09.08
*/

#pragma once

#include "CommonCode/RTOS abstraction layer/Thread.h"
#include "CommonCode/RTOS abstraction layer/Timer.h"

class CRemusReconImpl;
class CGuidanceImpl;

struct waypoint
{
	waypoint() : latitudeRadians(0.0), longitudeRadians(0.0) { }

	double latitudeRadians;
	double longitudeRadians;
};

struct directCommand
{
	directCommand() : thrustPercent(0.0), rudderPercent(0.0), sternPlanePercent(0.0) { }

	double thrustPercent;
	double rudderPercent;
	double sternPlanePercent;
};

class CGuidanceThread : public OAL::CThread
{
public:
	CGuidanceThread(CRemusReconImpl *pParent);
	virtual ~CGuidanceThread();

	bool initialize();
	bool startThread();
	void stopThread();
	void main();

	bool isInitalized(){ return m_bInitialized; }
	bool isRunning(){ return m_bRunning; }
	bool isInControl() { return m_bInControl; }

	CGuidanceImpl * getGuidanceImpl(){ return m_guidance; }

private:
	static const int c_period_tread = 100;				// Execution period in ms (>0)

	CRemusReconImpl *m_pParent;
	OAL::CTimer m_periodicTimer;

	CGuidanceImpl *m_guidance;

	bool m_bInitialized;
	bool m_bRunning;
	bool m_bInControl;

	int m_startLeg;
	int m_stopLeg;
	bool m_bDirectControlEnable;
	bool m_bDepthControlOnly;;
	bool m_bWaypointsEnable;
	bool m_bSpeedAsMsEnable;
	bool m_bAltitudeEnable;
	bool m_bAllowReconOnErrors;
	bool m_bOverrideStarted;
	int m_delayOverride;

	bool sendDirectionCommand();
	bool sendSpeedCommand();
	bool sendZCommand();
};