
#define BUFLEN 1000  //Max length of buffer

#include "MagnetometerThread.h"
#include "RemusReconImpl.h"
#include "GuidanceImpl.h"

#include "CommonCode/Log/LogManager.h"

CMagnetometerThread::CMagnetometerThread(CRemusReconImpl *pParent) :
OAL::CThread("CableTrackerPluginMagnetometerThread"),
//m_mutex("MagnetometerMutex"),
m_bInitialized(false), 
m_bRunning(false),
m_pParent(pParent),
m_port(8884),
m_crossTrackError(-1),
m_headingCable(-1),
m_messageNumber(0),
m_preciseMeasurements(false)
{
	;
}

CMagnetometerThread::~CMagnetometerThread()
{
	closesocket(m_socket);
	WSACleanup();
}

void CMagnetometerThread::main()
{
	m_pParent->storeEventString("MagnetometerThread: main() started.");

	while (!initialize())
	{
		std::cout << "CableTrackerPlugin::MagnetometerThread::initialize() failed. Retrying..." << std::endl;
		wait(1000);
	}

	std::cout << "CableTrackerPlugin::MagnetometerThread running..." << std::endl;

	ReconStatus_t reconState;

	int slen , recv_len;
	slen = sizeof(si_other) ;

	char buf[BUFLEN];
	bool firstMessageReceived = false;
	CAbsTime timeStart;

	//std::cout << "number of samples: "  << ssData.m_PingChanHeader->NumSamples << std::endl;
	
	while (!m_pParent->getSensorFusionThread()->isRunning())
	{
		sleep(50);
	}

	while( !m_ShutDown.wait(0) )
	{
		sleep(100);

		//clear the buffer by filling null, it might have previously received data
		memset(buf,'\0', BUFLEN);

		//simulation:
		double headingCable = m_pParent->getGuidanceThread()->getGuidanceImpl()->getWpCableHeading();
		double crossTrackError = m_pParent->getGuidanceThread()->getGuidanceImpl()->getWpCrossTrackError();

		m_pParent->getReconDriver()->getRemoteControlState(reconState);

		if (abs(crossTrackError) < 5 && CHPTimer::getInstance()->getAbsTime() > m_prevSampleTime + 333333)
		{
			m_headingCable = headingCable;
			m_crossTrackError = crossTrackError;
			m_messageNumber++;
			m_preciseMeasurements = true;
			m_prevSampleTime = CHPTimer::getInstance()->getAbsTime();
		}
		else if (crossTrackError < 30 && CHPTimer::getInstance()->getAbsTime() > m_prevSampleTime + 2000000 && false)
		{
			double signCrossTrack = crossTrackError > 0 ? 1.0 : -1.0;
			m_crossTrackError = signCrossTrack * 20;
			m_headingCable = headingCable;
			m_messageNumber++;
			m_preciseMeasurements = false;
			m_prevSampleTime = CHPTimer::getInstance()->getAbsTime();
		}

		//try to receive some data from udp socket
		if ((recv_len = recvfrom(m_socket, buf, BUFLEN, 0, (struct sockaddr *) &si_other, &slen)) == SOCKET_ERROR)
		{
			//printf("recvfrom() failed with error code : %d \n" , WSAGetLastError());
		}
		else 
		{
			char* pbuf = buf;
			char messageType;

			memcpy(&messageType, pbuf, sizeof(messageType));
			pbuf += sizeof(messageType);

			if (messageType == 'A')
			{
				m_pParent->getReconDriver()->getRemoteControlState(reconState);

				if (firstMessageReceived == false)
				{
					firstMessageReceived = true;
					timeStart = CHPTimer::getInstance()->getAbsTime();
				}

				double xAmp, yAmp, zAmp;
				double xPhase,yPhase, zPhase;
				double xMax, yMax, zMax;
				int rightLeft;
				double relHeading;
				double crossTrack;
				double depth;

				for (int i = 0; i < 2; i++)
				{
					memcpy(&xAmp, pbuf, sizeof(xAmp));
					pbuf += sizeof(xAmp);

					memcpy(&yAmp, pbuf, sizeof(yAmp));
					pbuf += sizeof(yAmp);

					memcpy(&zAmp, pbuf, sizeof(zAmp));
					pbuf += sizeof(zAmp);

					memcpy(&xPhase, pbuf, sizeof(xPhase));
					pbuf += sizeof(xPhase);

					memcpy(&yPhase, pbuf, sizeof(yPhase));
					pbuf += sizeof(yPhase);

					memcpy(&zPhase, pbuf, sizeof(zPhase));
					pbuf += sizeof(zPhase);

					memcpy(&xMax, pbuf, sizeof(xMax));
					pbuf += sizeof(xMax);

					memcpy(&yMax, pbuf, sizeof(yMax));
					pbuf += sizeof(yMax);

					memcpy(&zMax, pbuf, sizeof(zMax));
					pbuf += sizeof(zMax);

					storeMagnetometerData((unsigned char *) &xAmp, sizeof(xAmp));
					storeMagnetometerData((unsigned char *) &yAmp, sizeof(yAmp));
					storeMagnetometerData((unsigned char *) &zAmp, sizeof(zAmp));
					storeMagnetometerData((unsigned char *) &xPhase, sizeof(xPhase));
					storeMagnetometerData((unsigned char *) &yPhase, sizeof(yPhase));
					storeMagnetometerData((unsigned char *) &zPhase, sizeof(zPhase));
					storeMagnetometerData((unsigned char *) &xMax, sizeof(xMax));
					storeMagnetometerData((unsigned char *) &yMax, sizeof(yMax));
					storeMagnetometerData((unsigned char *) &zMax, sizeof(zMax));

					//std::cout << "MagnetometerThread: amp = " << xAmp << " " << yAmp << " " << zAmp << std::endl;
					//std::cout << "MagnetometerThread: phase = " << xPhase << " " << yPhase << " " << zPhase << std::endl;
					//std::cout << "MagnetometerThread: max = " << xMax << " " << yMax << " " << zMax << std::endl;

				}

				memcpy(&relHeading, pbuf, sizeof(relHeading));
				pbuf += sizeof(relHeading);

				memcpy(&rightLeft, pbuf, sizeof(rightLeft));	//right = 1, left = -1
				pbuf += sizeof(rightLeft);

				memcpy(&depth, pbuf, sizeof(depth));
				pbuf += sizeof(depth);

				memcpy(&crossTrack, pbuf, sizeof(crossTrack));
				pbuf += sizeof(crossTrack);

				memcpy(&m_messageNumber, pbuf, sizeof(m_messageNumber));
				pbuf += sizeof(m_messageNumber);

				m_headingCable = relHeading - reconState.Heading;
				m_crossTrackError = - crossTrack;

				//std::cout << "MagnetometerThread: relheading = " << relHeading << "  right/left =" << rightLeft << "  msg num = " << m_messageNumber << "  " << (CHPTimer::getInstance()->getAbsTime() - timeStart)/1000 << std::endl;
				//std::cout << "Depth: " << depth << "  crossTrack: " << crossTrack << std::endl;

				double m_messageNumberDouble = (double) m_messageNumber;
				double rightLeftDouble = (double) rightLeft;

				storeMagnetometerData((unsigned char *) &relHeading, sizeof(relHeading));
				storeMagnetometerData((unsigned char *) &rightLeftDouble, sizeof(rightLeftDouble));
				storeMagnetometerData((unsigned char *) &depth, sizeof(depth));
				storeMagnetometerData((unsigned char *) &crossTrack, sizeof(crossTrack));
				storeMagnetometerData((unsigned char *) &m_messageNumberDouble, sizeof(m_messageNumberDouble));


				//store some extra info
				double leg = (double) reconState.Leg;
				double wp = (double) m_pParent->getGuidanceThread()->getGuidanceImpl()->getCurrentWaypoint();

				storeMagnetometerData((unsigned char *) &reconState.Latitude, sizeof(reconState.Latitude));
				storeMagnetometerData((unsigned char *) &reconState.Longitude, sizeof(reconState.Longitude));
				storeMagnetometerData((unsigned char *) &reconState.Depth, sizeof(reconState.Depth));
				storeMagnetometerData((unsigned char *) &reconState.Altitude, sizeof(reconState.Altitude));
				storeMagnetometerData((unsigned char *) &reconState.Pitch, sizeof(reconState.Pitch));
				storeMagnetometerData((unsigned char *) &reconState.Roll, sizeof(reconState.Roll));
				storeMagnetometerData((unsigned char *) &reconState.Heading, sizeof(reconState.Heading));
				storeMagnetometerData((unsigned char *) &leg, sizeof(leg));
				storeMagnetometerData((unsigned char *) &wp, sizeof(wp));

			}
			else if (messageType == 'r')
			{
				//print details of the client/peer and the data received
				//printf("SideScanThread: Received packet from %s:%d\n", inet_ntoa(si_other.sin_addr), ntohs(si_other.sin_port));
				printf("SideScanThread: Data: %s\n" , buf);
			}
			else
			{
				std::cout << "MagnetometerThread: Undefined message received" << std::endl;
			}
		}

		/*
		char buffer[20] = "hallo";
		memset(buffer,'\0', BUFLEN);
		char* pBuffer = buffer;
		double num = 10;

		memcpy(pBuffer, &num, sizeof(num));
		pBuffer += sizeof(num);

		if (sendto(m_socket, buffer, sizeof(buffer), 0, (struct sockaddr*) &si_other, slen) == SOCKET_ERROR)
		{
		//printf("SideScanThread: sendto() failed with error code : %d\n" , WSAGetLastError());
		}
		else {
		//std::cout << "data sent " <<  << std::endl;
		}
		*/

	}

	std::cout << "CableTrackerPlugin::MagnetometerThread stopped..." << std::endl;

	// Log an event to the event log
	m_pParent->storeEventString("MagnetometerThread: main() stopped.");
}

bool CMagnetometerThread::initialize()
{
	if (m_bInitialized)
	{
		return m_bInitialized;
	}

	// Get udp server ip
	//GetConfigStringFromFile(m_pParent->getConfigFileName(), "Mangetometer", "IP", "127.0.0.1", m_IP, sizeof(m_IP));

	// Get udp port from .ini file
	m_port = GetConfigIntFromFile(m_pParent->getConfigFileName(), "Magnetometer", "Port", m_port);

	if (!initializeWinsock()){
		//closesocket(m_socket);
		WSACleanup();
		return m_bInitialized;
	}

	m_bInitialized = true;

	return m_bInitialized;
}

bool CMagnetometerThread::startThread()
{

	// Check if module has been initialized, if not, initialize module now
	if(!initialize())
	{
		// Log the event to the event log
		m_pParent->storeEventString("MagnetometerThread: Failed to initialize module.");
		std::cout << "ERROR: CableTrackerPlugin: MagnetometerThread::startThread() Unable to initialize WinSock" << std::endl;
		return false;
	}

	try
	{
		// Start thread
		start();
		m_bRunning = true;

		// Log an event to the event log
		m_pParent->storeEventString("MagnetometerThread: Thread started.");
		return m_bRunning;
	}
	catch ( ... )
	{
		m_pParent->storeEventString("MagnetometerThread: Exception when trying to start thread.");
		return false;
	}
}

void CMagnetometerThread::stopThread()
{
	// Shutdown thread
	(void) shutDown();

	// Log an event to the event log
	m_pParent->storeEventString("MagnetometerThread: Thread stopped.");

	m_bRunning = false;
}

bool CMagnetometerThread::initializeWinsock()
{
	WSADATA wsa;

	//Initialise winsock server
	printf("\nInitialising Winsock ...");
	if (WSAStartup(MAKEWORD(2,2),&wsa) != 0)
	{
		printf("MagnetometerThread: Failed. Error Code : %d",WSAGetLastError());
		return false;
	}
	printf("MagnetometerThread: Initialised.\n");

	//Create a socket
	if((m_socket = socket(AF_INET , SOCK_DGRAM , IPPROTO_UDP )) == INVALID_SOCKET)
	{
		printf("MagnetometerThread: Could not create socket : %d" , WSAGetLastError());
		return false;
	}
	printf("MagnetometerThread: Socket created.\n");

	//Prepare the sockaddr_in structure
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = INADDR_ANY;
	server.sin_port = htons( static_cast<u_short>(m_port) );

	u_long iMode=1;
	ioctlsocket(m_socket,FIONBIO,&iMode);

	//Bind
	if( bind(m_socket ,(struct sockaddr *)&server , sizeof(server)) == SOCKET_ERROR)
	{
		printf("MagnetometerThread: Bind failed with error code : %d" , WSAGetLastError());
		return false;
	}
	puts("MagnetometerThread: Bind done");

	return true;
}


// Store data using the plugin data storer
bool CMagnetometerThread::storeMagnetometerData(unsigned char *pData, size_t nDataLen)
{
	if (m_pParent->m_pMagnetometerDataStorer != NULL)
	{
		// Make storeObject ThreadSafe
		OAL::CCriticalSectionLock Lock( m_pParent->m_magnetometerStorageClient.getStorerCS() );

		// Reserve buffer for the data
		unsigned char* pbyStorageBuffer = m_pParent->m_magnetometerStorageClient.reserveBuffer((long) nDataLen );
		if (pbyStorageBuffer != NULL)
		{
			// Copy the data to the buffer
			memcpy(pbyStorageBuffer, pData, nDataLen);

			// Instruct the storage client to store the data to file
			m_pParent->m_magnetometerStorageClient.storeBuffer();
			return true;
		}
		else
		{
			CLogManager::error("CMagnetometerThread", "storeMagnetometerData", "Storage buffer == NULL.");
			return false;
		}
	}
	else
	{
		CLogManager::error("CMagnetometerThread", "storeMagnetometerData", "Data storer == NULL.");
		return false;
	}
}