
#define BUFLEN 60  //Max length of buffer

#include "NavThread.h"
#include "RemusReconImpl.h"

#include <stdio.h>
#include <sstream>
#include <string.h>

#include "CommonCode/Log/LogManager.h"

CNavThread::CNavThread(CRemusReconImpl *pParent) :
OAL::CThread("CableTrackerPluginNavThread"),
m_bInitialized(false), 
m_bRunning(false),
m_pParent(pParent),
m_port(8886)
{
	;
}

CNavThread::~CNavThread()
{
	closesocket(m_socket);
    WSACleanup();
}

void CNavThread::main()
{
	m_pParent->storeEventString("NavThread: main() started.");

	CNavSolutionData posData;

	while (!initialize())
	{
		std::cout << "CableTrackerPlugin::NavThread::initialize() failed. Retrying..." << std::endl;
		wait(1000);
	}

	std::cout << "CableTrackerPlugin::NavThread running..." << std::endl;

	int slen , recv_len;
	char buf[BUFLEN];

	slen = sizeof(si_other) ;

	while( !m_ShutDown.wait(0) )
	{
		// Wait for new data
		if(CPPDataBase::getInstance().wait(CPPDataBase::ePositionData, 100))
		{
			//std::cout << "New navigation data received" << std::endl;

			CPPDataBase::getInstance().read( CPPDataBase::ePositionData, posData);
			
			posDataStruct.m_nTimeStamp = posData.m_nTimeStamp;
			posDataStruct.m_dDepth = posData.m_dDepth;
			posDataStruct.m_dHeading = posData.m_dHeading;
			posDataStruct.m_dLatitude = posData.m_dLatitude;
			posDataStruct.m_dLongitude = posData.m_dLongitude;
			posDataStruct.m_dXVelocityB = posData.m_dXVelocityB;
			posDataStruct.m_dZAngRateB = posData.m_dZAngRateB;

			//clear the buffer by filling null, it might have previously received data
			memset(buf,'\0', BUFLEN);

			//try to receive some data
			if ((recv_len = recvfrom(m_socket, buf, BUFLEN, 0, (struct sockaddr *) &si_other, &slen)) == SOCKET_ERROR)
			{
				//printf("CNavThread:recvfrom() failed with error code : %d" , WSAGetLastError());
			}

			//print details of the client/peer and the data received
			//printf("CNavThread: Received packet from %s:%d\n", inet_ntoa(si_other.sin_addr), ntohs(si_other.sin_port));
			//printf("CNavThread: Data: %s\n" , buf);

			char buffer[sizeof(NavDataHeader)];
			memcpy(buffer, &posDataStruct, sizeof(NavDataHeader));

			if (sendto(m_socket, buffer, sizeof(NavDataHeader), 0, (struct sockaddr*) &si_other, slen) == SOCKET_ERROR)	//arg #3 var recv_len
			{
				//printf("CNavThread: sendto() failed with error code : %d\n" , WSAGetLastError());
			}
			else {
				//std::cout << "CNavThread: timestamp: " << posDataStruct.m_nTimeStamp << std::endl;
			}
		}
	}

	std::cout << "CableTrackerPlugin::NavThread stopped..." << std::endl;

	// Log an event to the event log
	m_pParent->storeEventString("NavThread: main() stopped.");
}

bool CNavThread::initialize()
{
	if (m_bInitialized)
	{
		return m_bInitialized;
	}

	// Get udp server ip
	//GetConfigStringFromFile(m_pParent->getConfigFileName(), "CableDetector", "PluginIP", "127.0.0.1", m_IP, sizeof(m_IP));

	// Get udp port from .ini file
	m_port = GetConfigIntFromFile(m_pParent->getConfigFileName(), "CableDetector", "NavPort", m_port);
	

	if (!initializeWinsock()){
		//closesocket(m_socket);
		WSACleanup();
		return m_bInitialized;
	}

	m_bInitialized = true;

	return m_bInitialized;
}

bool CNavThread::startThread()
{
	// Check if module has been initialized, if not, initialize module now
	if(!initialize())
	{
		// Log the event to the event log
		m_pParent->storeEventString("CNavThread: Failed to initialize module.");
		std::cout << "ERROR: CableTrackerPlugin: CNavThread::startThread() Unable to initialize WinSock" << std::endl;
		return false;
	}

	try
	{
		// Start thread
		start();
		m_bRunning = true;

		// Log an event to the event log
		m_pParent->storeEventString("CNavThread: Thread started.");
		return m_bRunning;
	}
	catch ( ... )
	{
		m_pParent->storeEventString("CNavThread: Exception when trying to start thread.");
		return false;
	}
}

void CNavThread::stopThread()
{
	// Shutdown thread
	(void) shutDown();

	// Log an event to the event log
	m_pParent->storeEventString("CNavThread: Thread stopped.");

	m_bRunning = false;
}

bool CNavThread::initializeWinsock()
{
	WSADATA wsa;
     
    //Initialise winsock server
    printf("\nInitialising Winsock ...");
    if (WSAStartup(MAKEWORD(2,2),&wsa) != 0)
    {
        printf("CNavThread: Failed. Error Code : %d",WSAGetLastError());
        return false;
    }
    printf("CNavThread: Initialised.\n");
     
    //Create a socket
    if((m_socket = socket(AF_INET , SOCK_DGRAM , 0 )) == INVALID_SOCKET)
    {
        printf("CNavThread: Could not create socket : %d" , WSAGetLastError());
		return false;
    }
    printf("CNavThread: Socket created.\n");
     
    //Prepare the sockaddr_in structure
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = INADDR_ANY;
    server.sin_port = htons( static_cast<u_short>(m_port) );

	u_long iMode=1;
	ioctlsocket(m_socket,FIONBIO,&iMode);
     
    //Bind
    if( bind(m_socket ,(struct sockaddr *)&server , sizeof(server)) == SOCKET_ERROR)
    {
        printf("CNavThread: Bind failed with error code : %d" , WSAGetLastError());
        return false;
    }
    puts("CNavThread: Bind done");

	return true;
}