/*
	RemusReconPluginImpl.h

	Copyright: 		Department of Marine Technology, NTNU
	Author: 		Petter Norgren
	Date created: 	2016.09.08
*/
#pragma once

// REMOVE WHEN DONE DEBUGGING
#define __DEBUGTHREAD

// REMOVE WHEN REMUS SIMULATOR IS AVAILABLE
#ifdef __DEBUGTHREAD
	class CDebugThread;
#endif

#include <PayloadSDK.h>
#include "PayloadInterface/payloadsensor.h"
#include "PayloadInterface/CPluginEventHandler.h"
#include "CommonCode/Data Storage/DataStorer.h"
#include "CommonCode/Data Storage/EventStorageClient.h"

#include "ReconMsgDefs.h"

//#pragma warning(push, 3)
#include "ReconDriver.h"
//#pragma warning(pop)

#include "GuidanceThread.h"
#include "SideScanThread.h"
#include "NavThread.h"
#include "MagnetometerThread.h"
#include "SensorFusionThread.h"

class CRemusReconImpl : public CPayloadSensor
{
public:
	CRemusReconImpl(void);
	virtual ~CRemusReconImpl(void);

	void activate ();
	int doCommands (const void * sendData, int len);

	void setDataStorer (CDataStorer &storer, int nStorerNo = 0);
	void setEventHandler (CPluginEventHandler *pEventHandler);
	void setEventStorer (CDataStorer &storer);
	void setMissionInfo (const char *path);

	const char * getCurrentFileName();
	const char * getName () const { return "CableTrackerPlugin"; }
	const char * getConfigFileName()const { return "CableTracker.ini"; }

	void stop ();
	void stopped ();
	void powerOn ();
	void powerOff ();
	void LoggingOff ();
	void LoggingOn ();	

	virtual void newLine (const char* tag);
	void sendReplyToOsCmdStatus (RCCmdResponseId eResponseId, RCCmdId eCmdID);
	void sendEventToOs (RCEventId eEventId);
	int payloadData (void* recvData, int len);

	bool storeData(unsigned char *pData, size_t nDataLen);
	void storeEventString(const char *pText);

	CDataStorer *m_pSideScanDataStorer;
	CStorageClient m_sideScanStorageClient;

	CDataStorer *m_pSensorFusionDataStorer;
	CStorageClient m_sensorFusionStorageClient;

	CDataStorer *m_pMagnetometerDataStorer;
	CStorageClient m_magnetometerStorageClient;
	
	CReconDriver * getReconDriver() { return m_pRCDriver; }
	CSideScanThread * getSideScanThread() { return m_pSideScanThread; }
	CNavThread * getNavThread() { return m_pNavThread; }
	CGuidanceThread * getGuidanceThread() { return m_pGuidanceThread; }
	CMagnetometerThread * getMagnetometerThread() { return m_pMagnetometerThread; }
	CSensorFusionThread * getSensorFusionThread() { return m_pSensorFusionThread; }

private:
	char * eventToString (RCEventId id);
	//bool connect();

	CEventStorageClient  m_EventLog;
	CPluginEventHandler *m_pEventHandler;
	CStorageClient		 m_dataStorageClient;
	CStorageClient		 m_eventStorageClient;
	CDataStorer			*m_pDataStorer;
	CDataStorer			*m_pEventDataStorer;

	CGuidanceThread		*m_pGuidanceThread;
	CReconDriver        *m_pRCDriver;
	CSideScanThread		*m_pSideScanThread;
	CNavThread			*m_pNavThread;
	CMagnetometerThread *m_pMagnetometerThread;
	CSensorFusionThread *m_pSensorFusionThread;

	// REMOVE WHEN DONE DEBUGGING
	#ifdef __DEBUGTHREAD
		CDebugThread	*m_debugThread;
	#endif
public:
	enum eEvents
	{
		eEventPowerOn = 128,
		eEventPowerOff,
		eEventReconPortOpened,
		eEventReconPortOpenFailed,
		eEventLoggingOn,
		eEventLoggingOff,
		eEventOperationOn,
		eEventOperationOff,
	};
};
