
#include "RemusReconImpl.h"
#include <Eigen/Dense>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <numeric>

//class CRemusReconImpl;

class CSensorFusion
{
public:
	CSensorFusion(CRemusReconImpl *pParent);
	virtual ~CSensorFusion();

	void initializeEkf();
	void ekfStep(const ReconStatus_t &reconState, double headingWP, double crossTrackErrWP);
	void multiRateEKF(const ReconStatus_t &reconState, double headingWP, double crossTrackErrWP);
	double getCableHeading(){return m_xHat(1,0); }
	double getCrossTrackError(){return m_xHat(0,0); } 

	bool useMultiRateEKF(){return m_bMultiRateEKF;}
private:
	CRemusReconImpl *m_pParent;
	
	bool m_bMultiRateEKF;
	int m_totSensors;
	int m_sensorsUsed;
	int m_numStates;
	
	bool m_bFuseSS;
	bool m_bFuseMag;
	bool m_bFuseAprDat;
	bool m_bWaitForInput;

	std::vector<bool> fuseSensor;
	std::vector<int> sensorNum;

	char m_matrixFolder[128];

	int m_bBadSetup;
	bool m_bHeaderCreated;

	Eigen::MatrixXd y;
	Eigen::MatrixXd m_xHat;

	Eigen::MatrixXd x_hat_prior;
	Eigen::MatrixXd P_prior;

	Eigen::MatrixXd H;
	Eigen::MatrixXd R;
	Eigen::MatrixXd Q;
	Eigen::MatrixXd E;
	//Eigen::MatrixXf abc;

	CAbsTime prevTimeStep;
	CAbsTime prevTimeStep_;
	//CAbsTime firstTimeStep;
	int m_prevSideScanUpdate;
	int m_prevMagnetometerUpdate;

	bool readMatrixFromFile(Eigen::MatrixXd& matrix,  const char *filename);
	void getMeasurements(const ReconStatus_t &reconState, double headingWP, double crossTrackErrWP);
	double generateGaussianNoise(double mu, double sigma);
	bool storeSensorFusionData(unsigned char *pData, size_t nDataLen);
	void createDataHeader();
};