#pragma once

#include "GuidanceImpl.h"

#include <vector>
#include <Eigen/Dense>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <numeric>

class CRemusReconImpl;

class CSensorFusionThread : public OAL::CThread
{
public:
	CSensorFusionThread(CRemusReconImpl *pParent);
	virtual ~CSensorFusionThread();


	void main();
	bool startThread();
	void stopThread();
	bool isRunning(){ return m_bRunning; }
	bool initializeEkf();
	void ekfStep();
	void multiRateEKF();
	double getCableHeading(){return m_xHat(1,0); }
	double getCrossTrackError(){return m_xHat(0,0); } 

	bool useMultiRateEKF(){return m_bMultiRateEKF;}
private:
	CRemusReconImpl *m_pParent;
	//OAL::CCriticalSection m_mutex;
	
	bool m_bMultiRateEKF;
	int m_totSensors;
	int m_sensorsUsed;
	int m_numStates;
	
	bool m_bFuseSS;
	bool m_bFuseMag;
	bool m_bFuseAprDat;
	bool m_bWaitForInput;
	int m_timeStep;

	std::vector<bool> fuseSensor;
	std::vector<int> sensorNum;

	char m_matrixFolder[128];

	bool m_bRunning;
	int m_bBadSetup;
	bool m_bHeaderCreated;

	Eigen::MatrixXd y;
	Eigen::MatrixXd m_xHat;

	Eigen::MatrixXd x_hat_prior;
	Eigen::MatrixXd P_prior;

	Eigen::MatrixXd H;
	Eigen::MatrixXd R;
	Eigen::MatrixXd Q;
	Eigen::MatrixXd E;
	//Eigen::MatrixXf abc;

	CAbsTime prevTimeStep;
	CAbsTime prevTimeStep_;
	//CAbsTime firstTimeStep;
	unsigned int m_prevSideScanUpdate;
	unsigned int m_prevMagnetometerUpdate;
	unsigned int m_prevWpUpdate;

	bool readMatrixFromFile(Eigen::MatrixXd& matrix,  const char *filename);
	void getMeasurements();
	double generateGaussianNoise(double mu, double sigma);
	bool storeSensorFusionData(unsigned char *pData, size_t nDataLen);
	bool createDataHeader();
};