
#define BUFLEN 100  //Max length of buffer

#include "SideScanThread.h"
#include "RemusReconImpl.h"

#include "CommonCode/Log/LogManager.h"

CSideScanThread::CSideScanThread(CRemusReconImpl *pParent) :
OAL::CThread("CableTrackerPluginSideScanThread"),
m_bInitialized(false), 
m_bRunning(false),
m_pParent(pParent),
m_port(8887),
m_crossTrack(-1),
m_headingCable(-1),
m_messageNumber(0)
{
	;
}

CSideScanThread::~CSideScanThread()
{

	closesocket(m_socket);
    WSACleanup();

}

void CSideScanThread::main()
{
	m_pParent->storeEventString("SideScanThread: main() started.");

	CSideScanData ssData;

	while (!initialize())
	{
		std::cout << "CableTrackerPlugin::SideScanThread::initialize() failed. Retrying..." << std::endl;
		wait(1000);
	}

	std::cout << "CableTrackerPlugin::SideScanThread running..." << std::endl;

	int slen , recv_len;
	slen = sizeof(si_other) ;

	char buf[BUFLEN];

	std::cout << "Saving data to: " << m_pParent->m_pSideScanDataStorer->getCurrentFileName() << std::endl;

	//std::cout << "number of samples: "  << ssData.m_PingChanHeader->NumSamples << std::endl;

	while( !m_ShutDown.wait(0) )
	{
		//clear the buffer by filling null, it might have previously received data
		memset(buf,'\0', BUFLEN);

		//try to receive some data from udp socket
		if ((recv_len = recvfrom(m_socket, buf, BUFLEN, 0, (struct sockaddr *) &si_other, &slen)) == SOCKET_ERROR)
		{
			//printf("recvfrom() failed with error code : %d" , WSAGetLastError());
		}
		else 
		{
			//std::cout << buf << std::endl;
			char* pbuf = buf;
			char messageType;

			memcpy(&messageType, pbuf, sizeof(messageType));
			pbuf += sizeof(messageType);

			if (messageType == 'n')
			{
				memcpy(&m_messageNumber, pbuf, sizeof(m_messageNumber));
				pbuf += sizeof(m_messageNumber);

				memcpy(&m_crossTrack, pbuf, sizeof(m_crossTrack));
				pbuf += sizeof(m_crossTrack);

				memcpy(&m_headingCable, pbuf, sizeof(m_headingCable));
				pbuf += sizeof(m_headingCable);

				//std::cout << "SideScanThread: " << m_crossTrack << "   " << m_headingCable *180/3.1415<< "   " << std::endl;
			}
			else if (messageType == 'r')
			{
				//print details of the client/peer and the data received
				//printf("SideScanThread: Received packet from %s:%d\n", inet_ntoa(si_other.sin_addr), ntohs(si_other.sin_port));
				//printf("SideScanThread: Data: %s\n" , buf);
			}
			else if (messageType == 'm')
			{
				printf("SideScanThread: Magnetometer data: %s\n" , buf);
			}
			else
			{
				std::cout << "SideScanThread: Undefined message received" << std::endl;
			}
		}


		// Wait for new data from sidescan sonar
		if(CPPDataBase::getInstance().wait(CPPDataBase::eSideScanData, 100))
		{
			//std::cout << "New sidescan data received" << std::endl;

			CPPDataBase::getInstance().read( CPPDataBase::eSideScanData, ssData);

			storeSideScanData((unsigned char *) &ssData.m_PingHeader, SizeOfPingHeader);
			storeSideScanData((unsigned char *) &ssData.m_PingChanHeader[0], SizeOfPingChanHeader);
			storeSideScanData((unsigned char *) &ssData.m_PingChanHeader[1], SizeOfPingChanHeader);

			for( int j = 0; j < 2; j++)
            {
                for (unsigned int i = 0; i < ssData.m_data[j].size(); i++)
                {
                    storeSideScanData((unsigned char *) &ssData.m_data[j][i], sizeof(ssData.m_data[j][i]));
                }
            }

			char buffer[20000];
			char* pBuffer = buffer;

			memcpy(pBuffer, &ssData.m_nTimeStamp, sizeof(ssData.m_nTimeStamp));
			pBuffer += sizeof(ssData.m_nTimeStamp);

			memcpy(pBuffer, &ssData.m_PingHeader.Body.PingNumber, sizeof(ssData.m_PingHeader.Body.PingNumber));
			pBuffer += sizeof(ssData.m_PingHeader.Body.PingNumber);

			memcpy(pBuffer, &ssData.m_PingHeader.Body.Speed, sizeof(ssData.m_PingHeader.Body.Speed));
			pBuffer += sizeof(ssData.m_PingHeader.Body.Speed);

			memcpy(pBuffer, &ssData.m_PingHeader.Body.Altitude, sizeof(ssData.m_PingHeader.Body.Altitude));
			pBuffer += sizeof(ssData.m_PingHeader.Body.Altitude);

			CCTDData ctdData;
			m_pParent->getGuidanceThread()->getGuidanceImpl()->getCTDThread()->getData(ctdData);

			float soundVel = (float) ctdData.m_dSoundVelocity;	//ssData.m_PingHeader.Body.SoundVelocity

			memcpy(pBuffer, &soundVel, sizeof(soundVel));
            pBuffer += sizeof(soundVel);

            memcpy(pBuffer, &ssData.m_PingChanHeader[0].SecondsPerLine, sizeof(ssData.m_PingChanHeader[0].SecondsPerLine));
            pBuffer += sizeof(ssData.m_PingChanHeader[0].SecondsPerLine);

            memcpy(pBuffer, &ssData.m_PingChanHeader[1].SecondsPerLine, sizeof(ssData.m_PingChanHeader[1].SecondsPerLine));
            pBuffer += sizeof(ssData.m_PingChanHeader[1].SecondsPerLine);

			int ssArrayLength = ssData.m_data[0].size();

			memcpy(pBuffer, &ssArrayLength , sizeof(ssArrayLength));
			pBuffer += sizeof(ssArrayLength);

			for( int j = 0; j < 2; j++)
            {
                for (unsigned int i = 0; i < ssData.m_data[j].size(); i++)
                {
                    memcpy(pBuffer, &ssData.m_data[j][i], sizeof(ssData.m_data[j][i]));
                    pBuffer += sizeof(ssData.m_data[j][i]);
                }
            }

			if (sendto(m_socket, buffer, sizeof(buffer), 0, (struct sockaddr*) &si_other, slen) == SOCKET_ERROR)
			{
				//printf("SideScanThread: sendto() failed with error code : %d\n" , WSAGetLastError());
			}
			else {
				//std::cout << "SideScanThread: sidescan data sent with ping number " << ssData.m_PingHeader.Body.PingNumber << std::endl;
			}
		}
	}

	std::cout << "CableTrackerPlugin::SideScanThread stopped..." << std::endl;

	// Log an event to the event log
	m_pParent->storeEventString("SideScanThread: main() stopped.");
}

bool CSideScanThread::initialize()
{
	if (m_bInitialized)
	{
		return m_bInitialized;
	}

	// Get udp server ip
	//GetConfigStringFromFile(m_pParent->getConfigFileName(), "CableDetector", "PluginIP", "127.0.0.1", m_IP, sizeof(m_IP));

	// Get udp port from .ini file
	m_port = GetConfigIntFromFile(m_pParent->getConfigFileName(), "CableDetector", "SideScanPort", m_port);

	if (!initializeWinsock()){
		//closesocket(m_socket);
		WSACleanup();
		return m_bInitialized;
	}

	m_bInitialized = true;

	return m_bInitialized;
}

bool CSideScanThread::startThread()
{
	
	// Check if module has been initialized, if not, initialize module now
	if(!initialize())
	{
		// Log the event to the event log
		m_pParent->storeEventString("SideScanThread: Failed to initialize module.");
		std::cout << "ERROR: CableTrackerPlugin: SideScanThread::startThread() Unable to initialize WinSock" << std::endl;
		return false;
	}

	try
	{
		// Start thread
		start();
		m_bRunning = true;

		// Log an event to the event log
		m_pParent->storeEventString("SideScanThread: Thread started.");
		return m_bRunning;
	}
	catch ( ... )
	{
		m_pParent->storeEventString("SideScanThread: Exception when trying to start thread.");
		return false;
	}
}

void CSideScanThread::stopThread()
{
	// Shutdown thread
	(void) shutDown();

	// Log an event to the event log
	m_pParent->storeEventString("SideScanThread: Thread stopped.");

	m_bRunning = false;
}

bool CSideScanThread::initializeWinsock()
{
	WSADATA wsa;
     
    //Initialise winsock server
    printf("\nInitialising Winsock ...");
    if (WSAStartup(MAKEWORD(2,2),&wsa) != 0)
    {
        printf("SideScanThread: Failed. Error Code : %d",WSAGetLastError());
        return false;
    }
    printf("SideScanThread: Initialised.\n");
     
    //Create a socket
    if((m_socket = socket(AF_INET , SOCK_DGRAM , IPPROTO_UDP )) == INVALID_SOCKET)
    {
        printf("SideScanThread: Could not create socket : %d" , WSAGetLastError());
		return false;
    }
    printf("SideScanThread: Socket created.\n");
     
    //Prepare the sockaddr_in structure
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = INADDR_ANY;
    server.sin_port = htons( static_cast<u_short>(m_port) );

	u_long iMode=1;
	ioctlsocket(m_socket,FIONBIO,&iMode);
     
    //Bind
    if( bind(m_socket ,(struct sockaddr *)&server , sizeof(server)) == SOCKET_ERROR)
    {
        printf("SideScanThread: Bind failed with error code : %d" , WSAGetLastError());
        return false;
    }
    puts("SideScanThread: Bind done");

	return true;
}



// Store data using the plugin data storer
bool CSideScanThread::storeSideScanData(unsigned char *pData, size_t nDataLen)
{
	return false;

	if (m_pParent->m_pSideScanDataStorer != NULL)
	{
		// Make storeObject ThreadSafe
		OAL::CCriticalSectionLock Lock( m_pParent->m_sideScanStorageClient.getStorerCS() );

		// Reserve buffer for the data
		unsigned char* pbyStorageBuffer = m_pParent->m_sideScanStorageClient.reserveBuffer((long) nDataLen );
		if (pbyStorageBuffer != NULL)
		{
			// Copy the data to the buffer
			memcpy(pbyStorageBuffer, pData, nDataLen);

			// Instruct the storage client to store the data to file
			m_pParent->m_sideScanStorageClient.storeBuffer();
			return true;
		}
		else
		{
			CLogManager::error("CSideScanThread", "storeSideScanData", "Storage buffer == NULL.");
			return false;
		}
	}
	else
	{
		CLogManager::error("CSideScanThread", "storeSideScanData", "Data storer == NULL.");
		return false;
	}
}